﻿using System;
using System.Text;
using System.Web.Mvc;
using SpiceLogic.PayPalCtrlForWPS.Controls;

namespace Test_Mvc_Web.Controllers
{
    public class AcceptingDonationController : Controller
    {
        //
        // GET: /AcceptingDonation/

        public ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Handles the IPN.
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public string HandleIPN()
        {
            IPNHandler myIpnHandler = new IPNHandler
            {
                NotifyValidateDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            DonationIPNEventArgs e = myIpnHandler.GetDonationDataForMVC();
 
            #region ---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------

            StringBuilder myIpnStringBuilder = new StringBuilder();
            myIpnStringBuilder.AppendLine("Transaction ID : " + e.TransactionID);

            myIpnStringBuilder.AppendLine("Donation Number : " + e.DonationNumber);
            myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross);
            myIpnStringBuilder.AppendLine("Discount Amount : " + e.PaymentInfo.DiscountAmount);
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet);
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName);
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City);
            myIpnStringBuilder.AppendLine("Is this ipn resent ? : " + e.IPN.IsResent);
            myIpnStringBuilder.AppendLine("Is this ipn from SandBox ? : " + e.IPN.IsSandBox);
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine(e.IPN.ToString());

            System.IO.File.WriteAllText(HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt",
                e.TransactionID)), myIpnStringBuilder.ToString());

            #endregion

            if (e.IPN.Status == PayPalIPN.StatusCodes.Verified)
            {
                /*********************** Data Collection ******************/
                string donationName = e.DonationName;
                string donationNumber = e.DonationNumber;
                decimal donatedAmount = e.PaymentInfo.McGross;
                DateTime donationDateTime = e.PaymentInfo.PaymentDateTime;
                string TransactionID = e.TransactionID;

                // etc............. etc........

                // Send email to the donator "Thank you for your donation".
                // Send email to the webmaster about the payment.
                // Log the transaction to your database.
            }

            return string.Empty;
        }

        /// <summary>
        /// Payspal returned.
        /// </summary>
        /// <returns></returns>
        public ActionResult PayPalReturned()
        {
            PayPalReturnHandler myReturnHandler = new PayPalReturnHandler
            {
                PDTAuthenticationToken = "IoSLdmDHmJZ9K2K98zASAG4y_DNx_MwgLMgGrF2gzrCwRnNo3VlU_bAw2DK",
                NotifySyncDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            DonationReturnedEventArgs e = myReturnHandler.GetDonationDataForMVC();

            ViewData["pdtData"] = e.PDT.ToString();

            string greetingMsg = string.Format("Hello [{0}] !", e.AdditionalDataItems["Customer's Name"]);

            if (e.IsPaymentCancelled)
            {
                ViewData["isPaymentCancelled"] = true;
                greetingMsg += " Sorry, donation is cancelled";
            }
            else
            {
                ViewData["txnId"] = e.TransactionID;

                greetingMsg += "Thank you for the donation.";
                ViewData["pdtStatus"] = e.PDT.Status;
                ViewData["isPaymentCancelled"] = false;

                if (e.PDT.Status == PayPalPDT.StatusCodes.SUCCESS)
                {
                    ViewData["item_name"] = e.DonationName;
                    ViewData["item_number"] = e.DonationNumber;
                    ViewData["paid_amount"] = e.PaymentInfo.McGross;
                }
            }

            ViewData["returnMessage"] = greetingMsg;

            return View();
        }

        /// <summary>
        /// Retrieves the ipn.
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ActionResult RetrieveIpn(string txnId)
        {
            string filePathForValidIPN = HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt", txnId));
            if (System.IO.File.Exists(filePathForValidIPN))
            {
                ViewData["ipnData"] = System.IO.File.ReadAllText(filePathForValidIPN);
                System.IO.File.Delete(filePathForValidIPN);
            }
            else
                ViewData["ipnData"] = string.Format("The IPN for Txn ID : {0} is still not available. It is natural for Sand box that the IPN arrives little longer than PayPal live site. Please try after a while.", txnId);

            ViewData["txnId"] = txnId;

            return View("PayPalReturned");
        }
    }
}
