﻿using System;
using System.Text;
using System.Web.Mvc;
using SpiceLogic.PayPalCtrlForWPS.Controls;
using SpiceLogic.PayPalCtrlForWPS.Services;
using Test_Mvc_Web.Models;

namespace Test_Mvc_Web.Controllers
{
    public class SellingSingleItemController : Controller
    {
        /// <summary>
        /// Indexes this instance.
        /// </summary>
        /// <returns></returns>
        public ActionResult Index()
        {
            ViewData["Item Name"] = "Beautiful Landscape";
            ViewData["Quantity"] = 2;
            ViewData["Price"] = 4.32;
            return View();
        }

        /// <summary>
        /// Updates the quantity.
        /// </summary>
        /// <param name="quantity">The quantity.</param>
        /// <param name="itemName">Name of the item.</param>
        /// <param name="price">The price.</param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult UpdateQuantity(int quantity, string itemName, decimal price)
        {
            ViewData["Quantity"] = quantity;
            ViewData["Item Name"] = itemName;
            ViewData["Price"] = price;
            return View("Index");
        }

        /// <summary>
        /// Handles the IPN.
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public string HandleIPN()
        {
            IPNHandler myIpnHandler = new IPNHandler
            {
                NotifyValidateDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            BuyNowIPNEventArgs e = myIpnHandler.GetBuyNowDataForMVC();

            #region ---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------

            StringBuilder myIpnStringBuilder = new StringBuilder();
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " +
                                          e.AdditionalDataItems["Customer's name"]);
            myIpnStringBuilder.AppendLine("Transaction ID : " + e.TransactionID);
            myIpnStringBuilder.AppendLine("Item Number : " + e.ItemInfo.ItemNumber);
            myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross);
            myIpnStringBuilder.AppendLine("Discount Amount : " + e.PaymentInfo.DiscountAmount);
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet);
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName);
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City);
            myIpnStringBuilder.AppendLine("Is this ipn resent ? : " + e.IPN.IsResent);
            myIpnStringBuilder.AppendLine("Is this ipn from SandBox ? : " + e.IPN.IsSandBox);
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine(e.IPN.ToString());

            System.IO.File.WriteAllText(HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myIpnStringBuilder.ToString());

            #endregion

            #region --------------------------- SALE Processing Logic -----------------------------------

            if (e.IPN.Status == PayPalIPN.StatusCodes.Verified)
            {
                /**********************************************************************************************************
                 * Now, we will follow a procedure to verify the data received from IPN and process the SALipnData. If you      *
                 * are using Encrypted Payment Button, then, you do not need to follow this verification process. Just    *
                 * always check  if (ipnData.IPN.Status == PayPalIPN.StatusCodes.Verified). Interested to learn about Encrypted *
                 * Payment generation ? It is very easy when you got this control. Just read from this documentation      *
                 * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21               *
                 * ********************************************************************************************************/


                /******************************** STEP 1 : Log the IPN record to Database **********************
                 *                                                                                             *
                 *   Assume that you have a method named mySalesInfoAdapter.Insert(...) to store the IPN data  *
                 *   in your databasipnData.                                                                         *
                 *                                                                                             *
                 *   mySalesInfoAdapter.Insert(ipnData.PaymentDateTime, ipnData.TransactionID, ipnData.ItemInfo.ItemName,        *
                 *   ipnData.ItemInfo.ItemNumber, ipnData.BuyerInfo.FirstName, ipnData.BuyerInfo.LastName,                       *
                 *   ipnData.BuyerInfo.PayerEmail, ipnData.PaymentInfo.PaymentGross, ipnData.ItemInfo.Quantity,                  *
                 *   ipnData.PaymentInfo.PaymentTypipnData.ToString(), ipnData.TransactionTypipnData.ToString());                      *
                 *                                                                                             *
                 ********************************************* x ***********************************************/

                /******************************** STEP 2 : Checking completed payment *****************************/

                if (e.PaymentInfo.PaymentStatus != IPNPDTPaymentInfo.PaymentStatusValues.Completed)
                {
                    string messageBody = string.Format("Incomplete Payment : Transaction ID = {0}", e.TransactionID);

                    /*****************************************************************************************************
                     * This SendEmail method is not a magical method. This SendEmail method uses your server's SMTP with *
                     * the name "localhost". It is not guranted to be working if your server's SMTP is not configured or *
                     * not allowed connection with 'localhost'. But usually, it works for most of the situation. If that *
                     * does not work for you, you can use the other Overload of this SendEmail method which takes SMTP   *
                     * Server name, SMTP User/Password as method argument.                                               *
                     * ***************************************************************************************************/

                    SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                 Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                 string.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName),
                                                 "Incomplete Payment notified from PayPal", messageBody, false);
                    return string.Empty;
                }

                /******************************** STEP 3: Checking Valid Payment Receiver Email *******************/

                // Constants.MyBusinessEmail is defined in file named Constants.cs in your App_Code folder. Please 
                // check that file and change the email address to the TEST Seller account email address, prior to 
                // this test.

                if ((e.BusinessEmail != Constants.MyBusinessEmail) ||
                    (e.ReceiverEmail != Constants.MyBusinessEmail))
                {
                    string messageBody = string.Format("Fraud Attempt : Transaction ID = {0}", e.TransactionID);
                    SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                 Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                 string.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName),
                                                 "Fraud attempt detected.", messageBody, false);
                    return string.Empty;
                }

                /********************************************** x *************************************************/

                /*** STEP 4: Checking if the itemNumber exists in Products Database and if the payment was correct. ***/
                bool ifPurchasedProductIsAValidProduct = true;
                // Or false, get this information by querying your databasipnData.

                if (ifPurchasedProductIsAValidProduct)
                {
                    // Item found in the products database, so check the price from your databasipnData.
                    // then check if the price matches with the (ipnData.PaymentInfo.McGross x ipnData.ItemInfo.Quantity) valuipnData. 

                    decimal productsOriginalPrice = 0.05m; // Retrieved from your databasipnData.

                    if (e.PaymentInfo.McGross >= (e.ItemInfo.Quantity * productsOriginalPrice))
                    {
                        // Payment was correct, not manipulated.,

                        /************* STEP 5 : Finally check if the Transaction id is an old onipnData. ******************/

                        // Now query your database to find out if the transaction id was used beforipnData.

                        int totalTxnIDCount = 1;
                        // The total number of transaction id found with the value 'ipnData.TransactionID'

                        /*******************************************************************************************************
                         * Check if (totalTxnIDCount > 1) rather than (totalTxnIDCount > 0)                                    *
                         * Because when IPN is notified, you already recorded this transaction with this ID. this query will   *
                         * always return 1 instead of 0.                                                                       *
                         *******************************************************************************************************/
                        if (totalTxnIDCount > 1)
                        {
                            // Possible fraud attempt. The Fraud person used an OLD Transaction ID to post this IPN. Notify Web Admin.

                            SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                         Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                         e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName,
                                                         "Duplicate Transaction ID",
                                                         @"Records indicate that this transaction is fraud, the customer is 
                           using an old transaction id to simulate this purchasipnData., Transaction ID:"
                                                         + e.TransactionID, false);

                            /******************************************************************************************* 
                             * Note: This case is not always truipnData.                                                     *
                             * For example, if your customer paid you using eCheck then the payment is not             *
                             * Completed but a transaction id is issued. Next time when the payment is completed,      *
                             * PayPal sends another IPN with the same Transaction ID, so if you count like this,       *
                             * your script will treat that completed notification as a fraud attempt.                  *
                             * So, you better develop your customized logic to handle those kind of situations.        *
                             * We have just demonstrated the way how to go ahead and explore the feature of this control.
                             * *****************************************************************************************/
                        }
                        else
                        {
                            /*************************************** VALID **************************************
                             * At this state, all test has been passed and so you may deliver the product.       *
                             * **********************************************************************************/

                            string downloadLink = SpiceHelperMethods.MapUrl("~/Content/images/Landscape_Purchased.jpg");

                            /**************************************************************************************
                             * This is just a demo. You may require sophisticated logic for providing a secured   *
                             * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
                             * delivering secure download link. Here is the URL:                                  *
                             * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
                             * ************************************************************************************/

                            string message =
                                string.Format(
                                    @"Dear Sir, Thank you very much for your payment 
                     for the product ""{0}"". Transaction ID :{1}",
                                    e.ItemInfo.ItemName, e.TransactionID);
                            message +=
                                string.Format("{0}Here is the download link of your product : <a href=\"{1}\">{2}</a>",
                                              Environment.NewLine, downloadLink, downloadLink);
                            SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail,
                                                         string.Format("{0} {1}", e.BuyerInfo.FirstName,
                                                                       e.BuyerInfo.LastName),
                                                         Constants.MyBusinessEmail, "Admin - MyCompany",
                                                         "Purchase Receipt",
                                                         message, false);

                            /***************************************************************************************
                             * You may also want to provide more information about the payment into your receipt.  *
                             * Just explore the class members of the event argument class and you will find a lot  *
                             * of clue about how to design your e store logic.                                     *
                             * *************************************************************************************/
                        }
                    }
                    else
                    {
                        /**********************************************************************************************
                         * The price was manipulated by a FRAUD attempt. notify the admin by email about this attempt *
                         * and do not deliver the product.                                                            *
                         **********************************************************************************************/

                        SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                     Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                     e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName,
                                                     "Fraud Attempt detected",
                                                     @"Fraud attempt detected : The payment_gross value is different " +
                                                     "from the expected amount for the transaction id :" +
                                                     e.TransactionID, false);
                    }
                }
                else
                {
                    //Let the customer know that the provided itemNumber does not exists.
                    SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail,
                                                 e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName,
                                                 Constants.MyBusinessEmail, "Admin - MyCompany",
                                                 "Invalid Item Number",
                                                 string.Format(
                                                     "Dear Sir, you paid for an invalid item number. Transaction ID :{0}",
                                                     e.TransactionID), false);
                }
            }

            #endregion

            /***************************************** note: Conclusion *************************************************
         * So, how you found the validation steps ? NOT Easy ? Hard ? Confusing and not comfortable to maintain ?   *
         * Too many steps to follow ? If so, then, you would love to use Encrypted Payments. Please read from this  *
         * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21. Without this   *
         * control it was never easier to generate Encrypted Payment Button on the fly (programmatically) with      *
         * strongly typed Object Oriented Manner.                                                                   *
         ************************************************************************************************************/
            
            return string.Empty;
        }

        /// <summary>
        /// Pays the pal returned.
        /// </summary>
        /// <returns></returns>
        public ActionResult PayPalReturned()
        {
            PayPalReturnHandler myReturnHandler = new PayPalReturnHandler
            {
                PDTAuthenticationToken = "IoSLdmDHmJZ9K2K98zASAG4y_DNx_MwgLMgGrF2gzrCwRnNo3VlU_bAw2DK",
                NotifySyncDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            BuyNowReturnedEventArgs e = myReturnHandler.GetBuyNowDataForMVC();

            ViewData["pdtData"] = e.PDT.ToString();

            string greetingMsg = string.Format("Hello [{0}] !", e.AdditionalDataItems["Customer's Name"]);

            if (e.IsPaymentCancelled)
            {
                ViewData["isPaymentCancelled"] = true;
                greetingMsg += " Sorry, payment is cancelled";
            }
            else
            {
                ViewData["txnId"] = e.TransactionID;

                greetingMsg += "Thank you for the payment.";
                ViewData["pdtStatus"] = e.PDT.Status;
                ViewData["isPaymentCancelled"] = false;

                if (e.PDT.Status == PayPalPDT.StatusCodes.SUCCESS)
                {
                    ViewData["item_name"] = e.ItemInfo.ItemName;
                    ViewData["item_number"] = e.ItemInfo.ItemNumber;
                    ViewData["paid_amount"] = e.PaymentInfo.McGross;
                    ViewData["quantity"] = e.ItemInfo.Quantity;
                    ViewData["buyerLastName"] = e.BuyerInfo.LastName;
                    ViewData["addressCity"] = e.BuyerInfo.ShippingAddress.City;
                    ViewData["charset"] = e.CharSet;
                    ViewData["paymentStatus"] = e.PaymentInfo.PaymentStatus;
                    ViewData["discount"] = e.PaymentInfo.DiscountAmount;
                }
            }

            ViewData["returnMessage"] = greetingMsg;

            return View();
        }

        /// <summary>
        /// Retrieves the ipn.
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ActionResult RetrieveIpn(string txnId)
        {
            string filePathForValidIPN = HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt", txnId));
            if (System.IO.File.Exists(filePathForValidIPN))
            {
                ViewData["ipnData"] = System.IO.File.ReadAllText(filePathForValidIPN);
                System.IO.File.Delete(filePathForValidIPN);
            }
            else
                ViewData["ipnData"] = string.Format("The IPN for Txn ID : {0} is still not available. It is natural for Sand box that the IPN arrives little longer than PayPal live site. Please try after a while.", txnId);

            ViewData["txnId"] = txnId;

            return View("PayPalReturned");
        }
    }
}
