﻿using System;
using System.Text;
using System.Web.Mvc;
using SpiceLogic.PayPalCtrlForWPS.Controls;
using SpiceLogic.PayPalCtrlForWPS.Services;
using Test_Mvc_Web.Models;

namespace Test_Mvc_Web.Controllers
{
    public class Upload3rdPartyShoppingCartController : Controller
    {
        //
        // GET: /Upload3rdPartyShoppingCart/

        public ActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Handles the IPN.
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public string HandleIPN()
        {
            IPNHandler myIpnHandler = new IPNHandler
            {
                NotifyValidateDestination =
                    FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            ShoppingCartIPNEventArgs e = myIpnHandler.GetShoppingCartDataForMVC();

            #region ---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------

            StringBuilder myBuilder = new StringBuilder();
            myBuilder.AppendLine("Transaction Id : " + e.TransactionID);
            myBuilder.AppendLine("Cart Name : " + e.AdditionalDataItems["Cart Name"]);
            myBuilder.AppendLine("Total Cart Items : " + e.ShoppingCartItems.Count);
            myBuilder.AppendLine("Name of the Second Item : " + e.ShoppingCartItems[1].ItemName);
            myBuilder.AppendLine("McGross : " + e.PaymentInfo.McGross);
            myBuilder.AppendLine("McFeee : " + e.PaymentInfo.McFee);
            myBuilder.AppendLine("Total number of items sold : " + e.GetTotalQuantities());
            myBuilder.AppendLine("Name of Second Item : " + e.ShoppingCartItems[1].ItemName);
            myBuilder.AppendLine("Price of 3rd Item : " + e.ShoppingCartItems[2].McGross);
            myBuilder.AppendLine();
            myBuilder.AppendLine();
            myBuilder.AppendLine(e.IPN.ToString());

            System.IO.File.WriteAllText(HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myBuilder.ToString());

            #endregion

            if (e.IPN.Status == PayPalIPN.StatusCodes.Verified)
            {
                /**********************************************************************************************************
                 * Now, we will follow a procedure to verify the data received from IPN and process the SALE. If you      *
                 * are using Encrypted Payment Button, then, you do not need to follow this verification process. Just    *
                 * always check  if (e.IPN.Status == PayPalIPN.StatusCodes.Verified). Interested to learn about Encrypted *
                 * Payment generation ? It is very easy when you got this control. Just read from this documentation      *
                 * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21               *
                 **********************************************************************************************************/

                /******************************** STEP 1 : Checking completed payment *****************************/

                if (e.PaymentInfo.PaymentStatus != IPNPDTPaymentInfo.PaymentStatusValues.Completed)
                {
                    string messageBody = string.Format("Incomplete Payment : Transaction ID = {0}",
                                                       e.TransactionID);

                    /*****************************************************************************************************
                     * This SendEmail method is not a magical method. This SendEmail method uses your server's SMTP with *
                     * the name "localhost". It is not guranted to be working if your server's SMTP is not configured or *
                     * not allowed connection with 'localhost'. But usually, it works for most of the situation. If that *
                     * does not work for you, you can use the other Overload of this SendEmail method which takes SMTP   *
                     * Server name, SMTP User/Password as method argument.                                               *
                     * ***************************************************************************************************/

                    SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                 Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                 string.Format("{0} {1}", e.BuyerInfo.FirstName,
                                                               e.BuyerInfo.LastName),
                                                 "Incomplete Payment notified from PayPal", messageBody, false);
                    return string.Empty;
                }


                /******************************** STEP 3: Checking Valid Payment Receiver Email *******************/

                // Constants.MyBusinessEmail is defined in file named Constants.cs in your App_Code folder. Please 
                // check that file and change the email address to the TEST Seller account email address, prior to 
                // this test.

                if ((e.BusinessEmail != Constants.MyBusinessEmail) ||
                    (e.ReceiverEmail != Constants.MyBusinessEmail))
                {
                    string messageBody = string.Format("Fraud Attempt : Transaction ID = {0}", e.TransactionID);
                    SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                                 Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                                 string.Format("{0} {1}", e.BuyerInfo.FirstName,
                                                               e.BuyerInfo.LastName),
                                                 "Fraud attempt detected.", messageBody, false);
                    return string.Empty;
                }

                /******************************** STEP 4: Data Collection *****************************************/

                for (int i = 0; i < e.ShoppingCartItems.Count; i++)
                {
                    string itemName = e.ShoppingCartItems[i].ItemName;
                    string itemNumber = e.ShoppingCartItems[i].ItemNumber;
                    decimal mcGross = e.ShoppingCartItems[i].McGross;
                    decimal mcHandling = e.ShoppingCartItems[i].McHandling;
                    decimal mcShipping = e.ShoppingCartItems[i].McShipping;

                    //etc........ etc......
                }


                /*************************************** VALID **************************************
                 * At this state, all test has been passed and so you may deliver the product.       *
                 ************************************************************************************/

                string downloadLink = SpiceHelperMethods.MapUrl("~/Content/images/Landscape_Purchased.jpg");

                /**************************************************************************************
                 * This is just a demo. You may require sophisticated logic for providing a secured   *
                 * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
                 * delivering secure download link. Here is the URL:                                  *
                 * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
                 * ************************************************************************************/

                string message =
                    string.Format(
                        @"Dear Sir, Thank you very much for your payment 
                     for the products ""{0}"". Transaction ID :{1}",
                        e.AdditionalDataItems["Cart Name"], e.TransactionID);
                message +=
                    string.Format("{0}Here is the download link of your product : <a href=\"{1}\">{2}</a>",
                                  Environment.NewLine, downloadLink, downloadLink);
                SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail,
                                             string.Format("{0} {1}", e.BuyerInfo.FirstName,
                                                           e.BuyerInfo.LastName),
                                             Constants.MyBusinessEmail, "Admin - MyCompany",
                                             "Purchase Receipt",
                                             message, false);

                /***************************************************************************************
                 * You may also want to provide more information about the payment into your receipt.  *
                 * Just explore the class members of the event argument class and you will find a lot  *
                 * of clue about how to design your e store logic.                                     *
                 * *************************************************************************************/

                return string.Empty;
            }

            return string.Empty;
        }

        /// <summary>
        /// Pays the pal returned.
        /// </summary>
        /// <returns></returns>
        public ActionResult PayPalReturned()
        {
            PayPalReturnHandler myReturnHandler = new PayPalReturnHandler
            {
                PDTAuthenticationToken = "IoSLdmDHmJZ9K2K98zASAG4y_DNx_MwgLMgGrF2gzrCwRnNo3VlU_bAw2DK",
                NotifySyncDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox
            };

            ShoppingCartReturnedEventArgs e = myReturnHandler.GetShoppingCartDataForMVC();
            ViewData["pdtData"] = e.PDT.ToString();

            string greetingMsg = string.Format("Hello [{0}] !", e.AdditionalDataItems["Customer's Name"]);

            if (e.IsPaymentCancelled)
            {
                ViewData["isPaymentCancelled"] = true;
                greetingMsg += " Sorry, payment is cancelled";
            }
            else
            {
                ViewData["txnId"] = e.TransactionID;

                greetingMsg += "Thank you for the payment.";
                ViewData["pdtStatus"] = e.PDT.Status;
                ViewData["isPaymentCancelled"] = false;

                if (e.PDT.Status == PayPalPDT.StatusCodes.SUCCESS)
                {
                    ViewData["paid_amount"] = e.PaymentInfo.McGross;
                    ViewData["buyerLastName"] = e.BuyerInfo.LastName;
                    ViewData["addressCity"] = e.BuyerInfo.ShippingAddress.City;
                    ViewData["charset"] = e.CharSet;
                    ViewData["paymentStatus"] = e.PaymentInfo.PaymentStatus;
                    ViewData["discount"] = e.PaymentInfo.DiscountAmount;
                    ViewData["totalQuantitySold"] = e.GetTotalQuantities();
                    ViewData["secondItemName"] = e.ShoppingCartItems.Count >= 2 ? e.ShoppingCartItems[1].ItemName : "";
                    ViewData["priceOf3rdItem"] = e.ShoppingCartItems.Count >= 3 ? e.ShoppingCartItems[2].McGross : 0;
                }
            }

            ViewData["returnMessage"] = greetingMsg;
            return View();
        }

        /// <summary>
        /// Retrieves the ipn.
        /// </summary>
        /// <returns></returns>
        public ActionResult RetrieveIpn(string txnId)
        {
            string filePathForValidIPN = HttpContext.Request.MapPath(string.Format("~/App_Data/ipn_{0}.txt", txnId));
            if (System.IO.File.Exists(filePathForValidIPN))
            {
                ViewData["ipnData"] = System.IO.File.ReadAllText(filePathForValidIPN);
                System.IO.File.Delete(filePathForValidIPN);
            }
            else
                ViewData["ipnData"] = string.Format("The IPN for Txn ID : {0} is still not available. It is natural for Sand box that the IPN arrives little longer than PayPal live site. Please try after a while.", txnId);

            ViewData["txnId"] = txnId;

            return View("PayPalReturned");
        }
    }
}
