﻿Imports VB.Models
Imports SpiceLogic.PayPalCtrlForWPS.Services
Imports SpiceLogic.PayPalCtrlForWPS.Controls

Namespace Controllers
    Public Class AddToCartController
        Inherits Controller

        '
        ' GET: /AddToCart/

        Public Function Index() As ActionResult
            Return View()
        End Function

        ''' <summary>
        ''' Handles the IPN.
        ''' </summary>
        ''' <returns></returns>
        <HttpPost()> _
        Public Function HandleIPN() As String
            Dim myIpnHandler As New IPNHandler() With { _
                    .NotifyValidateDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox _
                    }

            Dim e As ShoppingCartIPNEventArgs = myIpnHandler.GetShoppingCartDataForMVC()

            '#Region "---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------"

            Dim myBuilder As New StringBuilder()
            myBuilder.AppendLine("Transaction Id : " + e.TransactionID)
            myBuilder.AppendLine("Cart Name : " + e.AdditionalDataItems("Cart Name"))
            myBuilder.AppendLine("Total Cart Items : " + e.ShoppingCartItems.Count.ToString())
            myBuilder.AppendLine("Name of the Second Item : " + e.ShoppingCartItems(1).ItemName)
            myBuilder.AppendLine("McGross : " + e.PaymentInfo.McGross.ToString())
            myBuilder.AppendLine("McFeee : " + e.PaymentInfo.McFee.ToString())
            myBuilder.AppendLine("Total number of items sold : " + e.GetTotalQuantities().ToString())
            myBuilder.AppendLine("Name of Second Item : " + e.ShoppingCartItems(1).ItemName)
            myBuilder.AppendLine("Price of 3rd Item : " + e.ShoppingCartItems(2).McGross.ToString())
            myBuilder.AppendLine()
            myBuilder.AppendLine()
            myBuilder.AppendLine(e.IPN.ToString())

            IO.File.WriteAllText(HttpContext.Request.MapPath(String.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myBuilder.ToString())

            '#End Region

            If e.IPN.Status = PayPalIPN.StatusCodes.Verified Then
                '*********************************************************************************************************
                '                 * Now, we will follow a procedure to verify the data received from IPN and process the SALE. If you      *
                '                 * are using Encrypted Payment Button, then, you do not need to follow this verification process. Just    *
                '                 * always check  if (e.IPN.Status == PayPalIPN.StatusCodes.Verified). Interested to learn about Encrypted *
                '                 * Payment generation ? It is very easy when you got this control. Just read from this documentation      *
                '                 * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21               *
                '                 *********************************************************************************************************


                '******************************* STEP 1 : Checking completed payment ****************************


                If e.PaymentInfo.PaymentStatus <> IPNPDTPaymentInfo.PaymentStatusValues.Completed Then
                    Dim messageBody As String = String.Format("Incomplete Payment : Transaction ID = {0}", e.TransactionID)

                    '****************************************************************************************************
                    '                     * This SendEmail method is not a magical method. This SendEmail method uses your server's SMTP with *
                    '                     * the name "localhost". It is not guranted to be working if your server's SMTP is not configured or *
                    '                     * not allowed connection with 'localhost'. But usually, it works for most of the situation. If that *
                    '                     * does not work for you, you can use the other Overload of this SendEmail method which takes SMTP   *
                    '                     * Server name, SMTP User/Password as method argument.                                               *
                    '                     * **************************************************************************************************


                    SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), "Incomplete Payment notified from PayPal", messageBody, _
                                                 False)
                    Return String.Empty
                End If


                '******************************* STEP 3: Checking Valid Payment Receiver Email ******************


                ' Configs.MyBusinessEmail is defined in file named Constants.cs in your App_Code folder. Please 
                ' check that file and change the email address to the TEST Seller account email address, prior to 
                ' this test.

                If (e.BusinessEmail <> Configs.MyBusinessEmail) OrElse (e.ReceiverEmail <> Configs.MyBusinessEmail) Then
                    Dim messageBody As String = String.Format("Fraud Attempt : Transaction ID = {0}", e.TransactionID)
                    SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), "Fraud attempt detected.", messageBody, _
                                                 False)
                    Return String.Empty
                End If

                '******************************* STEP 4: Data Collection ****************************************


                For i As Integer = 0 To e.ShoppingCartItems.Count - 1
                    Dim itemName As String = e.ShoppingCartItems(i).ItemName
                    Dim itemNumber As String = e.ShoppingCartItems(i).ItemNumber
                    Dim mcGross As Decimal = e.ShoppingCartItems(i).McGross
                    Dim mcHandling As Decimal = e.ShoppingCartItems(i).McHandling

                    'etc........ etc......
                    Dim mcShipping As Decimal = e.ShoppingCartItems(i).McShipping
                Next


                '************************************** VALID **************************************
                '                 * At this state, all test has been passed and so you may deliver the product.       *
                '                 ***********************************************************************************


                Dim downloadLink As String = SpiceHelperMethods.MapUrl("~/Content/images/Landscape_Purchased.jpg")

                '*************************************************************************************
                '                 * This is just a demo. You may require sophisticated logic for providing a secured   *
                '                 * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
                '                 * delivering secure download link. Here is the URL:                                  *
                '                 * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
                '                 * ***********************************************************************************


                Dim message As String = String.Format("Dear Sir, Thank you very much for your payment " & vbCr & vbLf & "                     for the products ""{0}"". Transaction ID :{1}", e.AdditionalDataItems("Cart Name"), e.TransactionID)
                message += String.Format("{0}Here is the download link of your product : <a href=""{1}"">{2}</a>", Environment.NewLine, downloadLink, downloadLink)
                SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), Configs.MyBusinessEmail, "Admin - MyCompany", "Purchase Receipt", message, _
                                             False)

                '**************************************************************************************
                '                 * You may also want to provide more information about the payment into your receipt.  *
                '                 * Just explore the class members of the event argument class and you will find a lot  *
                '                 * of clue about how to design your e store logic.                                     *
                '                 * ************************************************************************************


                Return String.Empty
            End If

            Return String.Empty
        End Function

        ''' <summary>
        ''' Pays the pal returned.
        ''' </summary>
        ''' <returns></returns>
        Public Function PayPalReturned() As ActionResult
            Dim myReturnHandler As New PayPalReturnHandler() With { _
                    .PDTAuthenticationToken = "IoSLdmDHmJZ9K2K98zASAG4y_DNx_MwgLMgGrF2gzrCwRnNo3VlU_bAw2DK", _
                    .NotifySyncDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox _
                    }

            Dim e As ShoppingCartReturnedEventArgs = myReturnHandler.GetShoppingCartDataForMVC()
            ViewData("pdtData") = e.PDT.ToString()

            Dim greetingMsg As String = String.Format("Hello [{0}] !", e.AdditionalDataItems("Customer's Name"))

            If e.IsPaymentCancelled Then
                ViewData("isPaymentCancelled") = True
                greetingMsg += " Sorry, payment is cancelled"
            Else
                ViewData("txnId") = e.TransactionID

                greetingMsg += "Thank you for the payment."
                ViewData("pdtStatus") = e.PDT.Status
                ViewData("isPaymentCancelled") = False

                If e.PDT.Status = PayPalPDT.StatusCodes.SUCCESS Then
                    ViewData("paid_amount") = e.PaymentInfo.McGross
                    ViewData("buyerLastName") = e.BuyerInfo.LastName
                    ViewData("addressCity") = e.BuyerInfo.ShippingAddress.City
                    ViewData("charset") = e.CharSet
                    ViewData("paymentStatus") = e.PaymentInfo.PaymentStatus
                    ViewData("discount") = e.PaymentInfo.DiscountAmount
                    ViewData("totalQuantitySold") = e.GetTotalQuantities()
                    ViewData("secondItemName") = If(e.ShoppingCartItems.Count >= 2, e.ShoppingCartItems(1).ItemName, "")
                    ViewData("priceOf3rdItem") = If(e.ShoppingCartItems.Count >= 3, e.ShoppingCartItems(2).McGross, 0)
                End If
            End If

            ViewData("returnMessage") = greetingMsg

            Return View()
        End Function

        ''' <summary>
        ''' Retrieves the ipn.
        ''' </summary>
        ''' <returns></returns>
        <HttpPost()> _
        Public Function RetrieveIpn(ByVal txnId As String) As ActionResult
            Dim filePathForValidIPN As String = HttpContext.Request.MapPath(String.Format("~/App_Data/ipn_{0}.txt", txnId))
            If IO.File.Exists(filePathForValidIPN) Then
                ViewData("ipnData") = IO.File.ReadAllText(filePathForValidIPN)
                IO.File.Delete(filePathForValidIPN)
            Else
                ViewData("ipnData") = String.Format("The IPN for Txn ID : {0} is still not available. It is natural for Sand box that the IPN arrives little longer than PayPal live site. Please try after a while.", txnId)
            End If

            ViewData("txnId") = txnId

            Return View("PayPalReturned")
        End Function

    End Class
End Namespace