﻿Imports SpiceLogic.PayPalCtrlForWPS.Services
Imports VB.Models
Imports SpiceLogic.PayPalCtrlForWPS.Controls

Namespace Controllers
    Public Class SellingSingleItemController
        Inherits Controller

        ''' <summary>
        ''' Indexes this instance.
        ''' </summary>
        ''' <returns></returns>
        Public Function Index() As ActionResult
            ViewData("Item Name") = "Beautiful Landscape"
            ViewData("Quantity") = 2
            ViewData("Price") = 4.32
            Return View()
        End Function

        ''' <summary>
        ''' Updates the quantity.
        ''' </summary>
        ''' <param name="quantity">The quantity.</param>
        ''' <param name="itemName">Name of the item.</param>
        ''' <param name="price">The price.</param>
        ''' <returns></returns>
        <HttpPost()> _
        Public Function UpdateQuantity(ByVal quantity As Integer, ByVal itemName As String, ByVal price As Decimal) As ActionResult
            ViewData("Quantity") = quantity
            ViewData("Item Name") = itemName
            ViewData("Price") = price
            Return View("Index")
        End Function

        ''' <summary>
        ''' Handles the IPN.
        ''' </summary>
        ''' <returns></returns>
        <HttpPost()> _
        Public Function HandleIPN() As String
            Dim myIpnHandler As New IPNHandler() With { _
             .NotifyValidateDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox _
            }

            Dim e As BuyNowIPNEventArgs = myIpnHandler.GetBuyNowDataForMVC()

            '#Region "---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------"

            Dim myIpnStringBuilder As New StringBuilder()
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " + e.AdditionalDataItems("Customer's name"))
            myIpnStringBuilder.AppendLine("Transaction ID : " + e.TransactionID)
            myIpnStringBuilder.AppendLine("Item Number : " + e.ItemInfo.ItemNumber)
            myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross.ToString())
            myIpnStringBuilder.AppendLine("Discount Amount : " + e.PaymentInfo.DiscountAmount.ToString())
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet)
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName)
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City)
            myIpnStringBuilder.AppendLine("Is this ipn resent ? : " + e.IPN.IsResent.ToString())
            myIpnStringBuilder.AppendLine("Is this ipn from SandBox ? : " + e.IPN.IsSandBox.ToString())
            myIpnStringBuilder.AppendLine()
            myIpnStringBuilder.AppendLine()
            myIpnStringBuilder.AppendLine(e.IPN.ToString())

            IO.File.WriteAllText(HttpContext.Request.MapPath(String.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myIpnStringBuilder.ToString())

            '#End Region

            '#Region "--------------------------- SALE Processing Logic -----------------------------------"

            If e.IPN.Status = PayPalIPN.StatusCodes.Verified Then
                '*********************************************************************************************************
                '                 * Now, we will follow a procedure to verify the data received from IPN and process the SALipnData. If you      *
                '                 * are using Encrypted Payment Button, then, you do not need to follow this verification process. Just    *
                '                 * always check  if (ipnData.IPN.Status == PayPalIPN.StatusCodes.Verified). Interested to learn about Encrypted *
                '                 * Payment generation ? It is very easy when you got this control. Just read from this documentation      *
                '                 * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21               *
                '                 * *******************************************************************************************************



                '******************************* STEP 1 : Log the IPN record to Database **********************
                '                 *                                                                                             *
                '                 *   Assume that you have a method named mySalesInfoAdapter.Insert(...) to store the IPN data  *
                '                 *   in your databasipnData.                                                                         *
                '                 *                                                                                             *
                '                 *   mySalesInfoAdapter.Insert(ipnData.PaymentDateTime, ipnData.TransactionID, ipnData.ItemInfo.ItemName,        *
                '                 *   ipnData.ItemInfo.ItemNumber, ipnData.BuyerInfo.FirstName, ipnData.BuyerInfo.LastName,                       *
                '                 *   ipnData.BuyerInfo.PayerEmail, ipnData.PaymentInfo.PaymentGross, ipnData.ItemInfo.Quantity,                  *
                '                 *   ipnData.PaymentInfo.PaymentTypipnData.ToString(), ipnData.TransactionTypipnData.ToString());                      *
                '                 *                                                                                             *
                '                 ********************************************* x **********************************************


                '******************************* STEP 2 : Checking completed payment ****************************


                If e.PaymentInfo.PaymentStatus <> IPNPDTPaymentInfo.PaymentStatusValues.Completed Then
                    Dim messageBody As String = String.Format("Incomplete Payment : Transaction ID = {0}", e.TransactionID)

                    '****************************************************************************************************
                    '                     * This SendEmail method is not a magical method. This SendEmail method uses your server's SMTP with *
                    '                     * the name "localhost". It is not guranted to be working if your server's SMTP is not configured or *
                    '                     * not allowed connection with 'localhost'. But usually, it works for most of the situation. If that *
                    '                     * does not work for you, you can use the other Overload of this SendEmail method which takes SMTP   *
                    '                     * Server name, SMTP User/Password as method argument.                                               *
                    '                     * **************************************************************************************************


                    SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), "Incomplete Payment notified from PayPal", messageBody, _
                     False)
                    Return String.Empty
                End If

                '******************************* STEP 3: Checking Valid Payment Receiver Email ******************


                ' Configs.MyBusinessEmail is defined in file named Constants.cs in your App_Code folder. Please 
                ' check that file and change the email address to the TEST Seller account email address, prior to 
                ' this test.

                If (e.BusinessEmail <> Configs.MyBusinessEmail) OrElse (e.ReceiverEmail <> Configs.MyBusinessEmail) Then
                    Dim messageBody As String = String.Format("Fraud Attempt : Transaction ID = {0}", e.TransactionID)
                    SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), "Fraud attempt detected.", messageBody, _
                     False)
                    Return String.Empty
                End If

                '********************************************* x ************************************************


                '** STEP 4: Checking if the itemNumber exists in Products Database and if the payment was correct. **

                Dim ifPurchasedProductIsAValidProduct As Boolean = True
                ' Or false, get this information by querying your databasipnData.

                If ifPurchasedProductIsAValidProduct Then
                    ' Item found in the products database, so check the price from your databasipnData.
                    ' then check if the price matches with the (ipnData.PaymentInfo.McGross x ipnData.ItemInfo.Quantity) valuipnData. 

                    Dim productsOriginalPrice As Decimal = 0.05D
                    ' Retrieved from your databasipnData.
                    If e.PaymentInfo.McGross >= (e.ItemInfo.Quantity * productsOriginalPrice) Then
                        ' Payment was correct, not manipulated.,

                        '************ STEP 5 : Finally check if the Transaction id is an old onipnData. *****************


                        ' Now query your database to find out if the transaction id was used beforipnData.

                        Dim totalTxnIDCount As Integer = 1
                        ' The total number of transaction id found with the value 'ipnData.TransactionID'

                        '******************************************************************************************************
                        '                         * Check if (totalTxnIDCount > 1) rather than (totalTxnIDCount > 0)                                    *
                        '                         * Because when IPN is notified, you already recorded this transaction with this ID. this query will   *
                        '                         * always return 1 instead of 0.                                                                       *
                        '                         ******************************************************************************************************

                        If totalTxnIDCount > 1 Then
                            ' Possible fraud attempt. The Fraud person used an OLD Transaction ID to post this IPN. Notify Web Admin.


                            '****************************************************************************************** 
                            '                             * Note: This case is not always truipnData.                                                     *
                            '                             * For example, if your customer paid you using eCheck then the payment is not             *
                            '                             * Completed but a transaction id is issued. Next time when the payment is completed,      *
                            '                             * PayPal sends another IPN with the same Transaction ID, so if you count like this,       *
                            '                             * your script will treat that completed notification as a fraud attempt.                  *
                            '                             * So, you better develop your customized logic to handle those kind of situations.        *
                            '                             * We have just demonstrated the way how to go ahead and explore the feature of this control.
                            '                             * ****************************************************************************************

                            SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName, "Duplicate Transaction ID", "Records indicate that this transaction is fraud, the customer is " & vbCr & vbLf & "                           using an old transaction id to simulate this purchasipnData., Transaction ID:" + e.TransactionID, _
                             False)
                        Else
                            '************************************** VALID **************************************
                            '                             * At this state, all test has been passed and so you may deliver the product.       *
                            '                             * *********************************************************************************


                            Dim downloadLink As String = SpiceHelperMethods.MapUrl("~/Content/images/Landscape_Purchased.jpg")

                            '*************************************************************************************
                            '                             * This is just a demo. You may require sophisticated logic for providing a secured   *
                            '                             * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
                            '                             * delivering secure download link. Here is the URL:                                  *
                            '                             * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
                            '                             * ***********************************************************************************


                            Dim message As String = String.Format("Dear Sir, Thank you very much for your payment " & vbCr & vbLf & "                     for the product ""{0}"". Transaction ID :{1}", e.ItemInfo.ItemName, e.TransactionID)
                            message += String.Format("{0}Here is the download link of your product : <a href=""{1}"">{2}</a>", Environment.NewLine, downloadLink, downloadLink)

                            '**************************************************************************************
                            '                             * You may also want to provide more information about the payment into your receipt.  *
                            '                             * Just explore the class members of the event argument class and you will find a lot  *
                            '                             * of clue about how to design your e store logic.                                     *
                            '                             * ************************************************************************************

                            SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail, String.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName), Configs.MyBusinessEmail, "Admin - MyCompany", "Purchase Receipt", message, _
                             False)
                        End If
                    Else
                        '*********************************************************************************************
                        '                         * The price was manipulated by a FRAUD attempt. notify the admin by email about this attempt *
                        '                         * and do not deliver the product.                                                            *
                        '                         *********************************************************************************************


                        SpiceHelperMethods.SendEmail(Configs.MyBusinessEmail, Configs.MyBusinessEmail, e.BuyerInfo.PayerEmail, e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName, "Fraud Attempt detected", "Fraud attempt detected : The payment_gross value is different " + "from the expected amount for the transaction id :" + e.TransactionID, _
                         False)
                    End If
                Else
                    'Let the customer know that the provided itemNumber does not exists.
                    SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail, e.BuyerInfo.FirstName + " " + e.BuyerInfo.LastName, Configs.MyBusinessEmail, "Admin - MyCompany", "Invalid Item Number", String.Format("Dear Sir, you paid for an invalid item number. Transaction ID :{0}", e.TransactionID), _
                     False)
                End If
            End If

            '#End Region

            '**************************************** note: Conclusion *************************************************
            '         * So, how you found the validation steps ? NOT Easy ? Hard ? Confusing and not comfortable to maintain ?   *
            '         * Too many steps to follow ? If so, then, you would love to use Encrypted Payments. Please read from this  *
            '         * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21. Without this   *
            '         * control it was never easier to generate Encrypted Payment Button on the fly (programmatically) with      *
            '         * strongly typed Object Oriented Manner.                                                                   *
            '         ***********************************************************************************************************


            Return String.Empty
        End Function

        ''' <summary>
        ''' Pays the pal returned.
        ''' </summary>
        ''' <returns></returns>
        Public Function PayPalReturned() As ActionResult
            Dim myReturnHandler As New PayPalReturnHandler() With { _
             .PDTAuthenticationToken = "IoSLdmDHmJZ9K2K98zASAG4y_DNx_MwgLMgGrF2gzrCwRnNo3VlU_bAw2DK", _
             .NotifySyncDestination = FormSubmissionSettings.PostActionDestinations.PayPal_Sandbox _
            }

            Dim e As BuyNowReturnedEventArgs = myReturnHandler.GetBuyNowDataForMVC()

            ViewData("pdtData") = e.PDT.ToString()

            Dim greetingMsg As String = String.Format("Hello [{0}] !", e.AdditionalDataItems("Customer's Name"))

            If e.IsPaymentCancelled Then
                ViewData("isPaymentCancelled") = True
                greetingMsg += " Sorry, payment is cancelled"
            Else
                ViewData("txnId") = e.TransactionID

                greetingMsg += "Thank you for the payment."
                ViewData("pdtStatus") = e.PDT.Status
                ViewData("isPaymentCancelled") = False

                If e.PDT.Status = PayPalPDT.StatusCodes.SUCCESS Then
                    ViewData("item_name") = e.ItemInfo.ItemName
                    ViewData("item_number") = e.ItemInfo.ItemNumber
                    ViewData("paid_amount") = e.PaymentInfo.McGross
                    ViewData("quantity") = e.ItemInfo.Quantity
                    ViewData("buyerLastName") = e.BuyerInfo.LastName
                    ViewData("addressCity") = e.BuyerInfo.ShippingAddress.City
                    ViewData("charset") = e.CharSet
                    ViewData("paymentStatus") = e.PaymentInfo.PaymentStatus
                    ViewData("discount") = e.PaymentInfo.DiscountAmount
                End If
            End If

            ViewData("returnMessage") = greetingMsg

            Return View()
        End Function

        ''' <summary>
        ''' Retrieves the ipn.
        ''' </summary>
        ''' <returns></returns>
        <HttpPost()> _
        Public Function RetrieveIpn(ByVal txnId As String) As ActionResult
            Dim filePathForValidIPN As String = HttpContext.Request.MapPath(String.Format("~/App_Data/ipn_{0}.txt", txnId))
            If IO.File.Exists(filePathForValidIPN) Then
                ViewData("ipnData") = IO.File.ReadAllText(filePathForValidIPN)
                IO.File.Delete(filePathForValidIPN)
            Else
                ViewData("ipnData") = String.Format("The IPN for Txn ID : {0} is still not available. It is natural for Sand box that the IPN arrives little longer than PayPal live site. Please try after a while.", txnId)
            End If

            ViewData("txnId") = txnId

            Return View("PayPalReturned")
        End Function

    End Class
End Namespace