﻿using System;
using System.Globalization;
using System.IO;
using System.Text;
using SpiceLogic.PayPalCtrlForWPS.Controls;

public partial class Subscription : System.Web.UI.Page
{
    protected void Page_Load(object sender, EventArgs e)
    {

    }

    /********************************** Information about IPN_XXX Event *********************************************
    *                                                                                                                   *
    * IPN_Notified event is Raised when the customer Clicks Pay Now button in the PayPal website. This event is an      *
    * asynchronous event fired in the background as a Service Event. So, do not expect to see this                      *
    * event get fired even after your customer is returned back to your website. Sometimes it is delayed but most       *
    * of the time it is fired before PayPal_Returned event is fired. Consider this event as a pure Service Event,       *
    * do not perform any task that directly interacts with User Interface Controls, Session Variables, Cookies,         *
    * ViewState etc. When this event is fired, your customer wont see any effect of this event. For example, if you     *
    * want to set Text of a Label Control 'Thank you for your Payment' in this event, your customer will never see      *
    * that. Why ? Because, your customer is not directly responsible for firing this event, rather, PayPal Notification *
    * Service fires this event in a background thread, so whatever logic you run on User Interface Controls in this     *
    * event is rendered to that PayPal Notification Service which does not make any sense. For that reason, this control* 
    * SUPPRESS all HTML that could be rendered to the Event firer (PayPal Notification Service). Also, this control will* 
    * STOP firing the Page_Load event of the hosting page. WHY ? Maybe you placed some code in your Page_Load event     *
    * that directly interacts with Session Variables, Cookies etc or time consuming tasks like Executing lengthy SQL    *
    * queries which is unnecessary or sometimes harmful for IPN session. So, this control will bypass Page_Load event   *
    * in the IPN Session. What is the typical use of this event ? Ok, this event offers you a rich Event Argument Object*
    * which is loaded with the complete IPN variables in a strongly typed object oriented manner. You will get all      *
    * Transaction Data those are specific to a BuyNow button control is available from this Event Arg. So, you can use  *
    * this event to capture those data, store in your Database, send email to your customer, process a sale transaction,* 
    * update a member's role etc whatever that can be done in a background service. Yes, if you want to say 'Thank you  *
    * for your Payment' to your customer from your website just after the payment, PayPal_Returned event is implemented *
    * for that purpose. By the way, if you want to let this control fire the Page_Load event in the IPN session because *
    * you want to set IPN Settings programmatically from the Page_Load event, then, you have that option too. Set       *
    * PayPalIPN.EnablePageLoadEventInIPNSession = true and then, Page_Load event will be fired before IPN_Motified event* 
    * but you need to be very careful about using Page_Load event in this case. If you experience that your IPN_Notified* 
    * event is not getting fired, please check your ASPX markup and see if the IPN_Notified event handler mathod is     *
    * really wired up with your BuyNow button. Also, you should handle IPN_Exception event if you want to capture any   *
    * Exception thrown in your IPN_Notified event handler method. If you are testing a web application from your        *
    * "LocalHost" then this Event wont fire as PayPal cannot get this page by posting notification to a URL of LocalHost*
    * (127.0.0.1 or http://localhost:xxxx/). But if you host this page to a live server which has a real URL address    *
    * then the event will be fired and this method will be executed. If you test from LocalHost, and if you have any    *
    * default IPN URL setup in your PayPal profile, then, the IPN will be sent to that default URL in this case. Because* 
    * the control will detect that your test environment is LocalHost and so the control will not generate any IPN url  *
    * automatically and so, PayPal will use your Default IPN url from your profile to post the IPN. If the control      * 
    * generates the automatic IPN url when you are testing from localhost, then PayPal would try to post the IPN to your* 
    * localhost url which will eventually FAIL. So, PayPal would keep reposting to your non reachable URL many times and*
    * finally stop. But thats not all. PayPal may disable your IPN notification option for your account if you have many* 
    * non existing IPN url sent to PayPal. So, considering that side effect, this control will help you to avoid that problem.
    *                                                                                                                   *
    * *******************************************************************************************************************/


    /// <summary>
    /// IPN_Payment event is fired when a payment is made for the subcription.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="SubscriptionIPNPaymentEventArgs"/> instance containing the event data.</param>
    protected void SubscriptionButton_IPN_Payment(object sender, SubscriptionIPNPaymentEventArgs e)
    {
        StringBuilder myIpnStringBuilder = new StringBuilder();
        myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " +
                                      e.AdditionalDataItems["Customer's Name"]);
        myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID);
        myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber);
        myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross);
        myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName);
        myIpnStringBuilder.AppendLine("Charset : " + e.CharSet);
        myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName);
        myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City);
        myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event);
        myIpnStringBuilder.AppendLine();
        myIpnStringBuilder.AppendLine(e.IPN.ToString());
        File.WriteAllText(MapPath(string.Format("~/App_Data/suscr_ipn_payment_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString());
    }

    /// <summary>
    /// IPN_Subscription event is fired for any subscription related event like SignUp, Cancel, EndOfTerm etc.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">An <see cref="T:SpiceLogic.PayPalCtrlForWPS.SubscriptionIPNEventArgs">SubscriptionIPNEventArgs</see>
    /// that contains the event data.</param>
    protected void SubscriptionButton_IPN_Subscription(object sender, SubscriptionIPNEventArgs e)
    {
        if (e.SubscriptionInfo.Event == BasicIPNPDTSubscriptionInfo.SubscriptionEvents.SignUp)
        {
            StringBuilder myIpnStringBuilder = new StringBuilder();
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " +
                                          e.AdditionalDataItems["Customer's Name"]);
            myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID);
            myIpnStringBuilder.AppendLine("Recurring Time : " + e.SubscriptionInfo.RecurringTimes);
            myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber);
            myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName);
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet);
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName);
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City);
            myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event);
            myIpnStringBuilder.AppendLine("Subscription Regular Rate : " + e.SubscriptionInfo.McRegularRate);
            myIpnStringBuilder.AppendLine("Subscription Billing Cycle : " + e.SubscriptionInfo.BillingCycle + " " + e.SubscriptionInfo.BillingCycleUnit);
            myIpnStringBuilder.AppendLine("Trial Amount 1 : " + e.SubscriptionInfo.TrialAmount1);
            myIpnStringBuilder.AppendLine("Trial Cycle 1: " + e.SubscriptionInfo.TrialPeriod1 + " " + e.SubscriptionInfo.TrialPeriod1Unit);
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine(e.IPN.ToString());
            File.WriteAllText(MapPath(string.Format("~/App_Data/suscr_ipn_signup_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString());
        }
        else if (e.SubscriptionInfo.Event == BasicIPNPDTSubscriptionInfo.SubscriptionEvents.Cancel)
        {
            StringBuilder myIpnStringBuilder = new StringBuilder();
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " +
                                          e.AdditionalDataItems["Customer's Name"]);
            myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID);
            myIpnStringBuilder.AppendLine("Recurring Time : " + e.SubscriptionInfo.RecurringTimes);
            myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber);
            myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName);
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet);
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName);
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City);
            myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event);
            myIpnStringBuilder.AppendLine();
            myIpnStringBuilder.AppendLine(e.IPN.ToString());
            File.WriteAllText(MapPath(string.Format("~/App_Data/suscr_ipn_cancel_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString());
        }
     }

    /// <summary>
    /// Handles the Exception event of the SubscriptionButton_IPN control. This event is fired if there is any Exception
    /// thrown in your IPN_Notified event handler method or if there was any exception thrown in the notification
    /// verification phase of IPN. It is better to handle this event and keep your IPN_Notified event clean without
    /// using TRY-CATCH block there.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="SpiceLogic.PayPalCtrlForWPS.Core.IPNExceptionEventArgs"/> instance containing the event data.</param>
    protected void SubscriptionButton_IPN_Exception(object sender, IPNExceptionEventArgs e)
    {
        StringBuilder exceptionBuilder = new StringBuilder(e.IpnException.Message);
        exceptionBuilder.AppendLine("............................");
        exceptionBuilder.AppendLine("Stack Trace : ");
        exceptionBuilder.AppendLine(e.IpnException.StackTrace);
        File.WriteAllText(MapPath(string.Format("~/App_Data/subscription_ipn_err_{0}.txt", e.TransactionID)), exceptionBuilder.ToString());
    }

    /// <summary>
    /// Handles the Returned event of the SubscriptionButton_PayPal control. If you capture PDT (by providing the PDT Authentication
    /// token) you will get the same set of transaction data from it's event argument object that you get in IPN_Notified event.
    /// So, you can also process the post payment/subscription logic from this event. BUT, it is never a good idea to do so. WHY ? 
    /// Because, if your customer closes his/her browser before returning back to your website, then, this event will never get fired and the 
    /// post payment/subscription logic will not get executed. So, you should use a combination of IPN_Notified and PayPal_Returned event. Use 
    /// IPN_Notified event to proceed the core post payment/subscription logic (recording the transaction to database, send email, 
    /// update membership role etc) and use PayPal_Returned event to read that data back to your customer which was recorded 
    /// in the IPN session. OR, simply say 'Thank you for the Payment' to your customer. A Common question "What is the sequence
    /// of the IPN_Notified and PayPal_Returned event firing ?". The answer is not straight simple. Because, IPN_Notified event 
    /// is an Asynchronous Event fired by PayPal Notification Service depending on the queue of their IPN requests. That can be 
    /// instant as soon as the customer clicks 'PayNow' button in PayPal website, OR few minutes after the customer returned back
    /// to your website. So, most of the time, you will find that, IPN_Notified event is fired before PayPal_Returned event. But
    /// that sequence is not guaranteed. 
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="SubscriptionReturnedEventArgs"/> instance containing the event data.</param>
    protected void SubscriptionButton_PayPal_Returned(object sender, SubscriptionReturnedEventArgs e)
    {
        if (e.IsPaymentCancelled)
            MultiViewMyShop.SetActiveView(ViewPaymentCancelled);
        else
        {
            MultiViewMyShop.SetActiveView(ViewPaymentCompleted);
            lblTransactionID.Text = e.TransactionID;
            ViewState["SubscrId"] = e.SubscriptionInfo.SubscriberID;
            txtPDTString.Text = e.PDT.ToString();
            switch (e.PDT.Status)
            {
                case PayPalPDT.StatusCodes.SUCCESS:
                    lblThanks.Text = string.Format("Thank you {0} {1}! for your payment.",
                                                   e.BuyerInfo.FirstName, e.BuyerInfo.LastName);
                    lblSubscriptionName.Text = e.SubscriptionInfo.SubscriptionName;
                    lblSubscriptionNumber.Text = e.SubscriptionInfo.SubscriptionNumber;
                    lblResult.Text = e.PDT.Status.ToString();
                    lblPaymentAmount.Text = e.PaymentInfo.McGross.ToString(CultureInfo.InvariantCulture);
                    lblCustomerName.Text = e.AdditionalDataItems["Customer's Name"];
                    lblSubscriptionEvent.Text = e.SubscriptionInfo.Event.ToString();
                    lblSubscriberId.Text = e.SubscriptionInfo.SubscriberID;
                    break;
                case PayPalPDT.StatusCodes.NOT_Handling_PDT:
                    lblThanks.Text =
                        @"Thank you for the payment. As PDT is not being handled, 
                no transaction data is available. But you will get email about your purchased
                product receipt and shipping information.";
                    lblTransactionID.Text = e.TransactionID;
                    break;
                case PayPalPDT.StatusCodes.NO_PDT_Available:
                    lblThanks.Text =
                        @"Thank you signing up. The PDT is not available. Maybe because you had FREE Trial period 
for your Subscription. If there is a FREE Trial period, no payment transaction takes place at Subscription Sign up and so
when Customer is returned to your website from PayPal, PDT is not available.";
                    break;
            }
        }
    }

    /// <summary>
    /// Handles the Click event of the btnReadIPNString control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void btnReadIPNString_Click(object sender, EventArgs e)
    {
        if (ViewState["SubscrId"] == null)
        {
            lblIPNReadStatus.Text =
                "Looks like either PDT is not being handled or there was a FREE trial where payment was not made.";
            return;
        }

        string filePath = MapPath(string.Format("~/App_Data/suscr_ipn_signup_{0}.txt", ViewState["SubscrId"]));
        if (File.Exists(filePath))
        {
            txtIPNStr.Text = File.ReadAllText(filePath);
            try
            {
                File.Delete(filePath);
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder.";
            }
            catch
            {
                lblIPNReadStatus.Text = "IPN file could not be deleted.";
            }
        }
        else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. Try again.";
    }

    /// <summary>
    /// Handles the Click event of the btnReadIPNPaymentEventData control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void btnReadIPNPaymentEventData_Click(object sender, EventArgs e)
    {
        if (ViewState["SubscrId"] == null)
        {
            lblIPNReadStatus.Text =
                "Looks like either PDT is not being handled or there was a FREE trial where payment was not made.";
            return;
        }

        string filePath = MapPath(string.Format("~/App_Data/suscr_ipn_payment_{0}.txt", ViewState["SubscrId"]));
        if (File.Exists(filePath))
        {
            txtIPNStr.Text = File.ReadAllText(filePath);
            try
            {
                File.Delete(filePath);
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder.";
            }
            catch
            {
                lblIPNReadStatus.Text = "IPN file could not be deleted.";
            }
        }
        else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. Try again.";
    }

}
