﻿using System;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web.UI;
using SpiceLogic.PayPalCtrlForWPS.Controls;
using System.Xml;
using SpiceLogic.PayPalCtrlForWPS.Services;

public partial class UploadingItemsToPayPalFrom3rdPartyShoppingCart : Page
{
    protected void Page_Load(object sender, EventArgs e)
    {

    }

    /// <summary>
    /// Handles the Click event of the BuyNowButton1 control. Click event is fired only if a Click Event handler method is
    /// attached with the Click Event AND if the control is clicked. The Click event handler method will be executed before
    /// submitting form to PayPal. You can assign values to a PayPal Control's properties before submitting to PayPal Website.
    /// This approach is useful when you have a textbox in your page asking your buyer the quantity of the product he/she wants
    /// to buy and when the buyer click the BuyNow button, you want to update the Quantity information for this purchase. The
    /// control will POSTBACK ONLY IF you handle this Click Event. If you do not like POSTBACK for PayPal Control, then, set the
    /// property values of this control from Page_Load event or some other PostBack ASP.NET Control like ImageButton, Button,
    /// LinkButton etc control's PostBack event and do not handle Click Event. If you do not handle Click event of this control,
    /// then, clicking on this BuyNow button will transfer your visitor to PayPal (or SandBox) site from the client side without
    /// Server Round Trip which is the most desirable scenario.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="eventArgs">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void UploadCompleteCartButton1_Click(object sender, EventArgs eventArgs)
    {
        XmlDocument myeBooksDoc = new XmlDocument();
        myeBooksDoc.Load(MapPath("~/App_Data/eBooks.xml"));
        XmlNodeList eBooks = myeBooksDoc.SelectNodes("/eBooks/book");
        if (eBooks != null)
        {
            foreach (XmlNode book in eBooks)
            {
                string itemName = book.Attributes != null ? book.Attributes["title"].Value : string.Empty;
                string itemNumber = book.Attributes != null ? book.Attributes["id"].Value : string.Empty;
                decimal amount = book.Attributes != null ? decimal.Parse(book.Attributes["price"].Value) : 0.01m;

                PayPalItem myCartItem = new PayPalItem(itemName, itemNumber, 1, amount);
                UploadCompleteCartButton1.PayPalCartItems.Add(myCartItem);
            }
        }
    }

    /********************************** Information about IPN_Notified Event *********************************************
     *                                                                                                                   *
     * IPN_Notified event is Raised when the customer Clicks Pay Now button in the PayPal website. This event is an      *
     * asynchronous event fired in the background as a Service Event. So, do not expect to see this                      *
     * event get fired even after your customer is returned back to your website. Sometimes it is delayed but most       *
     * of the time it is fired before PayPal_Returned event is fired. Consider this event as a pure Service Event,       *
     * do not perform any task that directly interacts with User Interface Controls, Session Variables, Cookies,         *
     * ViewState etc. When this event is fired, your customer wont see any effect of this event. For example, if you     *
     * want to set Text of a Label Control 'Thank you for your Payment' in this event, your customer will never see      *
     * that. Why ? Because, your customer is not directly responsible for firing this event, rather, PayPal Notification *
     * Service fires this event in a background thread, so whatever logic you run on User Interface Controls in this     *
     * event is rendered to that PayPal Notification Service which does not make any sense. For that reason, this control* 
     * SUPPRESS all HTML that could be rendered to the Event firer (PayPal Notification Service). Also, this control will* 
     * STOP firing the Page_Load event of the hosting page. WHY ? Maybe you placed some code in your Page_Load event     *
     * that directly interacts with Session Variables, Cookies etc or time consuming tasks like Executing lengthy SQL    *
     * queries which is unnecessary or sometimes harmful for IPN session. So, this control will bypass Page_Load event   *
     * in the IPN Session. What is the typical use of this event ? Ok, this event offers you a rich Event Argument Object*
     * which is loaded with the complete IPN variables in a strongly typed object oriented manner. You will get all      *
     * Transaction Data those are specific to a BuyNow button control is available from this Event Arg. So, you can use  *
     * this event to capture those data, store in your Database, send email to your customer, process a sale transaction,* 
     * update a member's role etc whatever that can be done in a background service. Yes, if you want to say 'Thank you  *
     * for your Payment' to your customer from your website just after the payment, PayPal_Returned event is implemented *
     * for that purpose. By the way, if you want to let this control fire the Page_Load event in the IPN session because *
     * you want to set IPN Settings programmatically from the Page_Load event, then, you have that option too. Set       *
     * PayPalIPN.EnablePageLoadEventInIPNSession = true and then, Page_Load event will be fired before IPN_Motified event* 
     * but you need to be very careful about using Page_Load event in this case. If you experience that your IPN_Notified* 
     * event is not getting fired, please check your ASPX markup and see if the IPN_Notified event handler mathod is     *
     * really wired up with your BuyNow button. Also, you should handle IPN_Exception event if you want to capture any   *
     * Exception thrown in your IPN_Notified event handler method. If you are testing a web application from your        *
     * "LocalHost" then this Event wont fire as PayPal cannot get this page by posting notification to a URL of LocalHost*
     * (127.0.0.1 or http://localhost:xxxx/). But if you host this page to a live server which has a real URL address    *
     * then the event will be fired and this method will be executed. If you test from LocalHost, and if you have any    *
     * default IPN URL setup in your PayPal profile, then, the IPN will be sent to that default URL in this case. Because* 
     * the control will detect that your test environment is LocalHost and so the control will not generate any IPN url  *
     * automatically and so, PayPal will use your Default IPN url from your profile to post the IPN. If the control      * 
     * generates the automatic IPN url when you are testing from localhost, then PayPal would try to post the IPN to your* 
     * localhost url which will eventually FAIL. So, PayPal would keep reposting to your non reachable URL many times and*
     * finally stop. But thats not all. PayPal may disable your IPN notification option for your account if you have many* 
     * non existing IPN url sent to PayPal. So, considering that side effect, this control will help you to avoid that problem.
     *                                                                                                                   *
     * *******************************************************************************************************************/

    /// <summary>
    /// Handles the Notified event of the UploadCompleteCartButton1_IPN control. IPN URL needs to be an URL where PayPal 
    /// can post data from PayPal Website. If you are testing this web application from your "LocalHost" then this Event 
    /// wont fire as PayPal can not get this page by posting notification to a URL of LocalHost. But if you host this page 
    /// to a live server which has a real URL address then the event will be fired and this method will be executed.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="ShoppingCartIPNEventArgs"/> instance containing the event data.</param>
    protected void UploadCompleteCartButton1_IPN_Notified(object sender, ShoppingCartIPNEventArgs e)
    {
        #region ---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------

        StringBuilder myBuilder = new StringBuilder();
        myBuilder.AppendLine("Transaction Id : " + e.TransactionID);
        myBuilder.AppendLine("Cart Name : " + e.AdditionalDataItems["Cart Name"]);
        myBuilder.AppendLine("Total Cart Items : " + e.ShoppingCartItems.Count);
        myBuilder.AppendLine("Name of the Second Item : " + e.ShoppingCartItems[1].ItemName);
        myBuilder.AppendLine("McGross : " + e.PaymentInfo.McGross);
        myBuilder.AppendLine("McFeee : " + e.PaymentInfo.McFee);
        myBuilder.AppendLine("Total number of items sold : " + e.GetTotalQuantities());
        myBuilder.AppendLine("Name of Second Item : " + e.ShoppingCartItems[1].ItemName);
        myBuilder.AppendLine("Price of 3rd Item : " + e.ShoppingCartItems[2].McGross);
        myBuilder.AppendLine();
        myBuilder.AppendLine();
        myBuilder.AppendLine(e.IPN.ToString());

        File.WriteAllText(MapPath(string.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myBuilder.ToString());

        #endregion

        if (e.IPN.Status == PayPalIPN.StatusCodes.Verified)
        {
            /**********************************************************************************************************
             * Now, we will follow a procedure to verify the data received from IPN and process the SALE. If you      *
             * are using Encrypted Payment Button, then, you do not need to follow this verification process. Just    *
             * always check  if (e.IPN.Status == PayPalIPN.StatusCodes.Verified). Interested to learn about Encrypted *
             * Payment generation ? It is very easy when you got this control. Just read from this documentation      *
             * link: http://documentation.spicelogic.com/docs/ASP-NET-PayPal-Control/Getting-Started-21               *
             **********************************************************************************************************/

            /******************************** STEP 1 : Checking completed payment *****************************/

            if (e.PaymentInfo.PaymentStatus != IPNPDTPaymentInfo.PaymentStatusValues.Completed)
            {
                string messageBody = string.Format("Incomplete Payment : Transaction ID = {0}", e.TransactionID);

                /*****************************************************************************************************
                 * This SendEmail method is not a magical method. This SendEmail method uses your server's SMTP with *
                 * the name "localhost". It is not guranted to be working if your server's SMTP is not configured or *
                 * not allowed connection with 'localhost'. But usually, it works for most of the situation. If that *
                 * does not work for you, you can use the other Overload of this SendEmail method which takes SMTP   *
                 * Server name, SMTP User/Password as method argument.                                               *
                 * ***************************************************************************************************/

                SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                             Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                             string.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName),
                                             "Incomplete Payment notified from PayPal", messageBody, false);
                return;
            }


            /******************************** STEP 3: Checking Valid Payment Receiver Email *******************/

            // Constants.MyBusinessEmail is defined in file named Constants.cs in your App_Code folder. Please 
            // check that file and change the email address to the TEST Seller account email address, prior to 
            // this test.

            if ((e.BusinessEmail != Constants.MyBusinessEmail) ||
                (e.ReceiverEmail != Constants.MyBusinessEmail))
            {
                string messageBody = string.Format("Fraud Attempt : Transaction ID = {0}", e.TransactionID);
                SpiceHelperMethods.SendEmail(Constants.MyBusinessEmail,
                                             Constants.MyBusinessEmail, e.BuyerInfo.PayerEmail,
                                             string.Format("{0} {1}", e.BuyerInfo.FirstName, e.BuyerInfo.LastName),
                                             "Fraud attempt detected.", messageBody, false);
                return;
            }

            /******************************** STEP 4: Data Collection *****************************************/

            for (int i = 0; i < e.ShoppingCartItems.Count; i++)
            {
                string itemName = e.ShoppingCartItems[i].ItemName;
                string itemNumber = e.ShoppingCartItems[i].ItemNumber;
                decimal mcGross = e.ShoppingCartItems[i].McGross;
                decimal mcHandling = e.ShoppingCartItems[i].McHandling;
                decimal mcShipping = e.ShoppingCartItems[i].McShipping;

                //etc........ etc......
            }


            /*************************************** VALID **************************************
             * At this state, all test has been passed and so you may deliver the product.       *
             ************************************************************************************/

            string downloadLink = SpiceHelperMethods.MapUrl("~/Landscape_Purchased.jpg");

            /**************************************************************************************
             * This is just a demo. You may require sophisticated logic for providing a secured   *
             * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
             * delivering secure download link. Here is the URL:                                  *
             * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
             * ************************************************************************************/

            string message =
                    string.Format(
                        @"Dear Sir, Thank you very much for your payment 
                     for the products ""{0}"". Transaction ID :{1}", e.AdditionalDataItems["Cart Name"], e.TransactionID);
            message +=
                string.Format("{0}Here is the download link of your product : <a href=\"{1}\">{2}</a>",
                              Environment.NewLine, downloadLink, downloadLink);
            SpiceHelperMethods.SendEmail(e.BuyerInfo.PayerEmail,
                                         string.Format("{0} {1}", e.BuyerInfo.FirstName,
                                                       e.BuyerInfo.LastName),
                                         Constants.MyBusinessEmail, "Admin - MyCompany",
                                         "Purchase Receipt",
                                         message, false);

            /***************************************************************************************
             * You may also want to provide more information about the payment into your receipt.  *
             * Just explore the class members of the event argument class and you will find a lot  *
             * of clue about how to design your e store logic.                                     *
             * *************************************************************************************/
        }
    }

    /// <summary>
    /// Handles the Returned event of the UploadCompleteCartButton1_PayPal control. If you capture PDT (by providing the PDT Authentication
    /// token) you will get the same set of transaction data from it's event argument object that you get in IPN_Notified event.
    /// So, you can also process the SALE logic from this event. BUT, it is never a good idea to do so. WHY ? Because, if your 
    /// customer closes his/her browser before returning back to your website, then, this event will never get fired and the 
    /// SALE logic will not get executed. So, you should use a combination of IPN_Notified and PayPal_Returned event. Use 
    /// IPN_Notified event to proceed the core SALE processing logic (recording the transaction to database, send email, 
    /// update membership role etc) and use PayPal_Returned event to read that data back to your customer which was recorded 
    /// in the IPN session. OR, simply say 'Thank you for the Payment' to your customer. A Common question "What is the sequence
    /// of the IPN_Notified and PayPal_Returned event firing ?". The answer is not straight simple. Because, IPN_Notified event 
    /// is an Asynchronous Event fired by PayPal Notification Service depending on the queue of their IPN requests. That can be 
    /// instant as soon as the customer clicks 'PayNow' button in PayPal website, OR few minutes after the customer returned back
    /// to your website. So, most of the time, you will find that, IPN_Notified event is fired before PayPal_Returned event. But
    /// that sequence is not guaranteed. 
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="ShoppingCartIPNEventArgs"/> instance containing the event data.</param>
    protected void UploadCompleteCartButton1_PayPal_Returned(object sender, ShoppingCartReturnedEventArgs e)
    {
        if (e.IsPaymentCancelled)
        {
            lblHelloName.Text = "Hello " + UploadCompleteCartButton1.AdditionalDataItems["Customer's name"] + "!";
            MultiViewMyShop.SetActiveView(ViewPaymentCancelled);
        }
        else
        {
            ViewState["TransactionId"] = e.TransactionID; // We are storing this information in the ViewState because, we will use it in btnReadIPNData_Click method.
            lblResult.Text = e.PDT.Status.ToString();
            lblTransactionID.Text = e.TransactionID;
            MultiViewMyShop.SetActiveView(ViewPaymentCompleted);
            lblCartName.Text = e.AdditionalDataItems["Cart Name"];

            switch (e.PDT.Status)
            {
                case PayPalPDT.StatusCodes.SUCCESS:
                    lblBuyerEmail.Text = e.BuyerInfo.PayerEmail;
                    lblThanks.Text = string.Format("Thank you {0} {1}! for your payment.", e.BuyerInfo.FirstName, e.BuyerInfo.LastName);
                    lblQuantity.Text = e.GetTotalQuantities().ToString(CultureInfo.InvariantCulture);
                    lblPaidAmount.Text = e.PaymentInfo.McGross.ToString(CultureInfo.InvariantCulture);
                    lblNumberCartItems.Text = e.NumberOfCartItems.ToString(CultureInfo.InvariantCulture);

                    string downloadLink = SpiceHelperMethods.MapUrl("~/Landscape_Purchased.jpg");
                    lnkDownloadLandscape.NavigateUrl = downloadLink;

                    /**************************************************************************************
                     * This is just a demo. You may require sophisticated logic for providing a secured   *
                     * download link. You may check our product ASP.NET BLOB and Thumbnail Control for    *
                     * delivering secure download link. Here is the URL:                                  *
                     * http://www.spicelogic.com/Products/ASP-NET-BLOB-Thumbnail-Controls-2               *
                     * ************************************************************************************/

                    break;
                case PayPalPDT.StatusCodes.NOT_Handling_PDT:
                    lblThanks.Text = @"Thank you for the payment. As PDT is not being handled, 
         no transaction data is available. But you will get email about your purchased
         product receipt and shipping information.";
                    break;
            }
        }
    }

    /// <summary>
    /// Handles the Click event of the btnReadIPNData control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void btnReadIPNData_Click(object sender, EventArgs e)
    {
        if (Request.IsLocal)
            lblIPNReadStatus.Text =
                "As you are testing from LocalHost, naturally, IPN_Notified event was not fired,<br />and so, no file was written in the App_Data folder.";
        string filePath = MapPath(string.Format("~/App_Data/ipn_{0}.txt", ViewState["TransactionId"]));
        if (File.Exists(filePath))
        {
            txtIPNData.Text = File.ReadAllText(filePath);
            try
            {
                File.Delete(filePath);
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder.";
            }
            catch
            {
                lblIPNReadStatus.Text = "IPN file could not be deleted.";
            }
        }
        else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. <br />Click the 'Read the data..' button again after few more seconds. Sometimes it is delayed.";
    }
}
