﻿Imports System.IO
Imports SpiceLogic.PayPalCtrlForWPS.Controls

Partial Class DedicatedPayPalReturnHandler
    Inherits Page
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)

    End Sub

    ''' <summary>
    ''' Handles the PayPa_Returned event of the PayPalReturnHandler1 control. If you capture PDT (by providing the PDT Authentication
    ''' token) you will get the same set of transaction data from it's event argument object that you get in IPN_Notified event.
    ''' So, you can also process the SALE logic from this event. BUT, it is never a good idea to do so. WHY ? Because, if your 
    ''' customer closes his/her browser before returning back to your website, then, this event will never get fired and the 
    ''' SALE logic will not get executed. So, you should use a combination of IPN_Notified and PayPal_Returned event. Use 
    ''' IPN_Notified event to proceed the core SALE processing logic (recording the transaction to database, send email, 
    ''' update membership role etc) and use PayPal_Returned event to read that data back to your customer which was recorded 
    ''' in the IPN session. OR, simply say 'Thank you for the Payment' to your customer. A Common question "What is the sequence
    ''' of the IPN_Notified and PayPal_Returned event firing ?". The answer is not straight simple. Because, IPN_Notified event 
    ''' is an Asynchronous Event fired by PayPal Notification Service depending on the queue of their IPN requests. That can be 
    ''' instant as soon as the customer clicks 'PayNow' button in PayPal website, OR few minutes after the customer returned back
    ''' to your website. So, most of the time, you will find that, IPN_Notified event is fired before PayPal_Returned event. But
    ''' that sequence is not guaranteed. 
    ''' </summary>
    Protected Sub PayPalReturnHandler1_PayPal_Returned(ByVal sender As Object, ByVal e As DedicatedPayPalReturnedEventArgs)
        If e.IsPaymentCancelled Then
            MultiViewMyShop.SetActiveView(ViewPaymentCancelled)
        Else
            ViewState("TransactionId") = e.TransactionID
            ' We are storing this information in the ViewState because, we will use it in btnReadIPNData_Click method.
            lblResult.Text = e.PDT.Status.ToString()
            lblTransactionID.Text = e.TransactionID
            MultiViewMyShop.SetActiveView(ViewPaymentCompleted)

            Select Case e.PDT.Status
                Case PayPalPDT.StatusCodes.SUCCESS
                    lblBuyerEmail.Text = e.BuyerInfo.PayerEmail
                    lblThanks.Text = String.Format("Thank you {0} {1}! for your payment.", e.BuyerInfo.FirstName, e.BuyerInfo.LastName)
                    lblPaidAmount.Text = e.PaymentInfo.McGross.ToString()
                    lblNumberCartItems.Text = e.NumberOfCartItems.ToString()
                    Exit Select
                Case PayPalPDT.StatusCodes.NOT_Handling_PDT
                    lblThanks.Text = "Thank you for the payment. As PDT is not being handled, " & vbCr & vbLf & "         no transaction data is available. But you will get email about your purchased" & vbCr & vbLf & "         product receipt and shipping information."
                    Exit Select
            End Select
        End If
    End Sub

    ''' <summary>
    ''' Handles the Click event of the btnReadIPNData control.
    ''' </summary>
    Protected Sub btnReadIPNData_Click(ByVal sender As Object, ByVal e As EventArgs)
        If Request.IsLocal Then
            lblIPNReadStatus.Text = "As you are testing from LocalHost, naturally, IPN_Notified event was not fired,<br />and so, no file was written in the App_Data folder."
        End If
        Dim filePath As String = MapPath(String.Format("~/App_Data/dedicated_ipn_{0}.txt", ViewState("TransactionId")))
        If File.Exists(filePath) Then
            txtIPNData.Text = File.ReadAllText(filePath)
            Try
                File.Delete(filePath)
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder."
            Catch
                lblIPNReadStatus.Text = "IPN file could not be deleted."
            End Try
        Else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. <br />Click the 'Read the data..' button again after few more seconds. Sometimes it is delayed."
        End If
    End Sub
End Class