﻿Imports System
Imports System.IO
Imports System.Text
Imports SpiceLogic.PayPalCtrlForWPS.Controls

Partial Public Class Donation
    Inherits Page
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)

    End Sub

    '     ********************************* Information about IPN_Notified Event *********************************************
    '     *                                                                                                                   *
    '     * IPN_Notified event is Raised when the customer Clicks Pay Now button in the PayPal website. This event is an      *
    '     * asynchronous event fired in the background as a Service Event. So, do not expect to see this                      *
    '     * event get fired even after your customer is returned back to your website. Sometimes it is delayed but most       *
    '     * of the time it is fired before PayPal_Returned event is fired. Consider this event as a pure Service Event,       *
    '     * do not perform any task that directly interacts with User Interface Controls, Session Variables, Cookies,         *
    '     * ViewState etc. When this event is fired, your customer wont see any effect of this event. For example, if you     *
    '     * want to set Text of a Label Control 'Thank you for your Payment' in this event, your customer will never see      *
    '     * that. Why ? Because, your customer is not directly responsible for firing this event, rather, PayPal Notification *
    '     * Service fires this event in a background thread, so whatever logic you run on User Interface Controls in this     *
    '     * event is rendered to that PayPal Notification Service which does not make any sense. For that reason, this control* 
    '     * SUPPRESS all HTML that could be rendered to the Event firer (PayPal Notification Service). Also, this control will* 
    '     * STOP firing the Page_Load event of the hosting page. WHY ? Maybe you placed some code in your Page_Load event     *
    '     * that directly interacts with Session Variables, Cookies etc or time consuming tasks like Executing lengthy SQL    *
    '     * queries which is unnecessary or sometimes harmful for IPN session. So, this control will bypass Page_Load event   *
    '     * in the IPN Session. What is the typical use of this event ? Ok, this event offers you a rich Event Argument Object*
    '     * which is loaded with the complete IPN variables in a strongly typed object oriented manner. You will get all      *
    '     * Transaction Data those are specific to a BuyNow button control is available from this Event Arg. So, you can use  *
    '     * this event to capture those data, store in your Database, send email to your customer, process a sale transaction,* 
    '     * update a member's role etc whatever that can be done in a background service. Yes, if you want to say 'Thank you  *
    '     * for your Payment' to your customer from your website just after the payment, PayPal_Returned event is implemented *
    '     * for that purpose. By the way, if you want to let this control fire the Page_Load event in the IPN session because *
    '     * you want to set IPN Settings programmatically from the Page_Load event, then, you have that option too. Set       *
    '     * PayPalIPN.EnablePageLoadEventInIPNSession = true and then, Page_Load event will be fired before IPN_Motified event* 
    '     * but you need to be very careful about using Page_Load event in this case. If you experience that your IPN_Notified* 
    '     * event is not getting fired, please check your ASPX markup and see if the IPN_Notified event handler mathod is     *
    '     * really wired up with your BuyNow button. Also, you should handle IPN_Exception event if you want to capture any   *
    '     * Exception thrown in your IPN_Notified event handler method. If you are testing a web application from your        *
    '     * "LocalHost" then this Event wont fire as PayPal cannot get this page by posting notification to a URL of LocalHost*
    '     * (127.0.0.1 or http://localhost:xxxx/). But if you host this page to a live server which has a real URL address    *
    '     * then the event will be fired and this method will be executed. If you test from LocalHost, and if you have any    *
    '     * default IPN URL setup in your PayPal profile, then, the IPN will be sent to that default URL in this case. Because* 
    '     * the control will detect that your test environment is LocalHost and so the control will not generate any IPN url  *
    '     * automatically and so, PayPal will use your Default IPN url from your profile to post the IPN. If the control      * 
    '     * generates the automatic IPN url when you are testing from localhost, then PayPal would try to post the IPN to your* 
    '     * localhost url which will eventually FAIL. So, PayPal would keep re-posting to your non reachable URL many times and*
    '     * finally stop. But thats not all. PayPal may disable your IPN notification option for your account if you have many* 
    '     * non existing IPN url sent to PayPal. So, considering that side effect, this control will help you to avoid that problem.
    '     *                                                                                                                   *
    '     * ******************************************************************************************************************



    ''' <summary>
    ''' Handles the IPN_Notified event of the DonationButton1 control.
    ''' Please NOTE: IPN URL needs to be an URL where PayPal can post data from PayPal Website.
    ''' If you are testing a web application from your "LocalHost" then this Event wont fire as PayPal cannot get this
    ''' page by posting notification to a URL of LocalHost (127.0.0.1 or http://localhost:xxxx/). But if you host this 
    ''' page to a live server which has a real URL address then the event will be fired and this method will be executed.     
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="SpiceLogic.PayPalCtrlForWPS.BuyNowIPNEventArgs"/> instance containing the event data.</param>
    Protected Sub DonationButton1_IPN_Notified(ByVal sender As Object, ByVal e As DonationIPNEventArgs)
        '#Region "---------------------------- Just logging the IPN Session as a proof that IPN event was fired --------------"

        Dim myIpnStringBuilder As New StringBuilder()
        myIpnStringBuilder.AppendLine("Transaction ID : " + e.TransactionID)

        myIpnStringBuilder.AppendLine("Donation Number : " + e.DonationNumber)
        myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross.ToString())
        myIpnStringBuilder.AppendLine("Discount Amount : " + e.PaymentInfo.DiscountAmount.ToString())
        myIpnStringBuilder.AppendLine("Charset : " + e.CharSet)
        myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName)
        myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City)
        myIpnStringBuilder.AppendLine("Is this ipn resent ? : " + e.IPN.IsResent.ToString())
        myIpnStringBuilder.AppendLine("Is this ipn from SandBox ? : " + e.IPN.IsSandBox.ToString())
        myIpnStringBuilder.AppendLine()
        myIpnStringBuilder.AppendLine()
        myIpnStringBuilder.AppendLine(e.IPN.ToString())

        File.WriteAllText(MapPath(String.Format("~/App_Data/ipn_{0}.txt", e.TransactionID)), myIpnStringBuilder.ToString())

        '#End Region

        If e.IPN.Status = PayPalIPN.StatusCodes.Verified Then
            '********************** Data Collection *****************

            Dim donationName As String = e.DonationName
            Dim donationNumber As String = e.DonationNumber
            Dim donatedAmount As Decimal = e.PaymentInfo.McGross
            Dim donationDateTime As DateTime = e.PaymentInfo.PaymentDateTime

            ' etc............. etc........


            ' Send email to the donator "Thank you for your donation".
            ' Send email to the webmaster about the payment.
            ' Log the transaction to your database.
            Dim TransactionID As String = e.TransactionID
        End If
    End Sub

    ''' <summary>
    ''' Handles the Returned event of the DonationButton1_PayPal control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    Protected Sub DonationButton1_PayPal_Returned(ByVal sender As Object, ByVal e As DonationReturnedEventArgs)
        If e.IsPaymentCancelled Then
            MultiViewMyShop.SetActiveView(ViewPaymentCancelled)
        Else
            lblTransactionID.Text = e.TransactionID

            ViewState("TransactionId") = e.TransactionID
            ' We are storing this information in the ViewState because, we will use it in btnReadIPNData_Click method.
            lblResult.Text = e.PDT.Status.ToString()

            MultiViewMyShop.SetActiveView(ViewPaymentCompleted)

            Select Case e.PDT.Status
                Case PayPalPDT.StatusCodes.SUCCESS
                    lblThanks.Text = String.Format("Thank you {0} {1}! for your payment.", e.BuyerInfo.FirstName, e.BuyerInfo.LastName)

                    lblDonationName.Text = e.DonationName
                    lblDonationNumber.Text = e.DonationNumber
                    lblPaidAmount.Text = e.PaymentInfo.McGross.ToString()
                    Exit Select
                Case PayPalPDT.StatusCodes.NOT_Handling_PDT
                    lblThanks.Text = "Thank you for the payment. As PDT is not being handled, " & vbCr & vbLf & "         no transaction data is available. But you will get email about your purchased" & vbCr & vbLf & "         product receipt and shipping information."
                    Exit Select
            End Select
        End If
    End Sub

    ''' <summary>
    ''' Handles the Click event of the btnReadIPNData control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    Protected Sub btnReadIPNData_Click(ByVal sender As Object, ByVal e As EventArgs)
        If Request.IsLocal Then
            lblIPNReadStatus.Text = "As you are testing from LocalHost, naturally, IPN_Notified event was not fired,<br />and so, no file was written in the App_Data folder."
        End If
        Dim filePath As String = MapPath(String.Format("~/App_Data/ipn_{0}.txt", ViewState("TransactionId")))
        If File.Exists(filePath) Then
            txtIPNData.Text = File.ReadAllText(filePath)
            Try
                File.Delete(filePath)
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder."
            Catch
                lblIPNReadStatus.Text = "IPN file could not be deleted."
            End Try
        Else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. <br />Click the 'Read the data..' button again after few more seconds. Sometimes it is delayed."
        End If
    End Sub
End Class
