﻿Imports System
Imports System.IO
Imports System.Text
Imports SpiceLogic.PayPalCtrlForWPS.Controls

Partial Public Class Subscription
    Inherits Page
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)

    End Sub

    '********************************* Information about IPN_XXX Event ****************************************************
    ' *                                                                                                                   *
    ' * IPN_Notified event is Raised when the customer Clicks Pay Now button in the PayPal website. This event is an      *
    ' * asynchronous event fired in the background as a Service Event. So, do not expect to see this                      *
    ' * event get fired even after your customer is returned back to your website. Sometimes it is delayed but most       *
    ' * of the time it is fired before PayPal_Returned event is fired. Consider this event as a pure Service Event,       *
    ' * do not perform any task that directly interacts with User Interface Controls, Session Variables, Cookies,         *
    ' * ViewState etc. When this event is fired, your customer wont see any effect of this event. For example, if you     *
    ' * want to set Text of a Label Control 'Thank you for your Payment' in this event, your customer will never see      *
    ' * that. Why ? Because, your customer is not directly responsible for firing this event, rather, PayPal Notification *
    ' * Service fires this event in a background thread, so whatever logic you run on User Interface Controls in this     *
    ' * event is rendered to that PayPal Notification Service which does not make any sense. For that reason, this control* 
    ' * SUPPRESS all HTML that could be rendered to the Event firer (PayPal Notification Service). Also, this control will* 
    ' * STOP firing the Page_Load event of the hosting page. WHY ? Maybe you placed some code in your Page_Load event     *
    ' * that directly interacts with Session Variables, Cookies etc or time consuming tasks like Executing lengthy SQL    *
    ' * queries which is unnecessary or sometimes harmful for IPN session. So, this control will bypass Page_Load event   *
    ' * in the IPN Session. What is the typical use of this event ? Ok, this event offers you a rich Event Argument Object*
    ' * which is loaded with the complete IPN variables in a strongly typed object oriented manner. You will get all      *
    ' * Transaction Data those are specific to a BuyNow button control is available from this Event Arg. So, you can use  *
    ' * this event to capture those data, store in your Database, send email to your customer, process a sale transaction,* 
    ' * update a member's role etc whatever that can be done in a background service. Yes, if you want to say 'Thank you  *
    ' * for your Payment' to your customer from your website just after the payment, PayPal_Returned event is implemented *
    ' * for that purpose. By the way, if you want to let this control fire the Page_Load event in the IPN session because *
    ' * you want to set IPN Settings programmatically from the Page_Load event, then, you have that option too. Set       *
    ' * PayPalIPN.EnablePageLoadEventInIPNSession = true and then, Page_Load event will be fired before IPN_Motified event* 
    ' * but you need to be very careful about using Page_Load event in this case. If you experience that your IPN_Notified* 
    ' * event is not getting fired, please check your ASPX markup and see if the IPN_Notified event handler mathod is     *
    ' * really wired up with your BuyNow button. Also, you should handle IPN_Exception event if you want to capture any   *
    ' * Exception thrown in your IPN_Notified event handler method. If you are testing a web application from your        *
    ' * "LocalHost" then this Event wont fire as PayPal cannot get this page by posting notification to a URL of LocalHost*
    ' * (127.0.0.1 or http://localhost:xxxx/). But if you host this page to a live server which has a real URL address    *
    ' * then the event will be fired and this method will be executed. If you test from LocalHost, and if you have any    *
    ' * default IPN URL setup in your PayPal profile, then, the IPN will be sent to that default URL in this case. Because* 
    ' * the control will detect that your test environment is LocalHost and so the control will not generate any IPN url  *
    ' * automatically and so, PayPal will use your Default IPN url from your profile to post the IPN. If the control      * 
    ' * generates the automatic IPN url when you are testing from localhost, then PayPal would try to post the IPN to your* 
    ' * localhost url which will eventually FAIL. So, PayPal would keep reposting to your non reachable URL many times and*
    ' * finally stop. But thats not all. PayPal may disable your IPN notification option for your account if you have many* 
    ' * non existing IPN url sent to PayPal. So, considering that side effect, this control will help you to avoid that problem.
    ' *                                                                                                                   *
    ' * *******************************************************************************************************************



    ''' <summary>
    ''' IPN_Payment event is fired when a payment is made for the subcription.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    Protected Sub SubscriptionButton_IPN_Payment(ByVal sender As Object, ByVal e As SubscriptionIPNPaymentEventArgs)
        Dim myIpnStringBuilder As New StringBuilder()
        myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " + e.AdditionalDataItems("Customer's Name"))
        myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID)
        myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber)
        myIpnStringBuilder.AppendLine("Mc Gross : " + e.PaymentInfo.McGross.ToString())
        myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName)
        myIpnStringBuilder.AppendLine("Charset : " + e.CharSet)
        myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName)
        myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City)
        myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event.ToString())
        myIpnStringBuilder.AppendLine()
        myIpnStringBuilder.AppendLine(e.IPN.ToString())
        File.WriteAllText(MapPath(String.Format("~/App_Data/suscr_ipn_payment_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString())
    End Sub

    ''' <summary>
    ''' IPN_Subscription event is fired for any subscription related event like SignUp, Cancel, EndOfTerm etc.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' that contains the event data.</param>
    Protected Sub SubscriptionButton_IPN_Subscription(ByVal sender As Object, ByVal e As SubscriptionIPNEventArgs)
        If e.SubscriptionInfo.Event = BasicIPNPDTSubscriptionInfo.SubscriptionEvents.SignUp Then
            Dim myIpnStringBuilder As New StringBuilder()
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " + e.AdditionalDataItems("Customer's Name"))
            myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID)
            myIpnStringBuilder.AppendLine("Recurring Time : " + e.SubscriptionInfo.RecurringTimes.ToString())
            myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber)
            myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName)
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet)
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName)
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City)
            myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event.ToString())
            myIpnStringBuilder.AppendLine("Subscription Regular Rate : " + e.SubscriptionInfo.McRegularRate.ToString())
            myIpnStringBuilder.AppendLine("Subscription Billing Cycle : " + e.SubscriptionInfo.BillingCycle.ToString() + " " + e.SubscriptionInfo.BillingCycleUnit.ToString())
            myIpnStringBuilder.AppendLine("Trial Amount 1 : " + e.SubscriptionInfo.TrialAmount1.ToString())
            myIpnStringBuilder.AppendLine("Trial Cycle 1: " + e.SubscriptionInfo.TrialPeriod1.ToString() + " " + e.SubscriptionInfo.TrialPeriod1Unit.ToString())
            myIpnStringBuilder.AppendLine()
            myIpnStringBuilder.AppendLine(e.IPN.ToString())
            File.WriteAllText(MapPath(String.Format("~/App_Data/suscr_ipn_signup_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString())
        ElseIf e.SubscriptionInfo.Event = BasicIPNPDTSubscriptionInfo.SubscriptionEvents.Cancel Then
            Dim myIpnStringBuilder As New StringBuilder()
            myIpnStringBuilder.AppendLine("Additional Data Item -> Customer's Name = " + e.AdditionalDataItems("Customer's Name"))
            myIpnStringBuilder.AppendLine("Subscriber ID : " + e.SubscriptionInfo.SubscriberID)
            myIpnStringBuilder.AppendLine("Recurring Time : " + e.SubscriptionInfo.RecurringTimes.ToString())
            myIpnStringBuilder.AppendLine("Subscription Number : " + e.SubscriptionInfo.SubscriptionNumber)
            myIpnStringBuilder.AppendLine("Subscription Name : " + e.SubscriptionInfo.SubscriptionName)
            myIpnStringBuilder.AppendLine("Charset : " + e.CharSet)
            myIpnStringBuilder.AppendLine("Buyer Last Name : " + e.BuyerInfo.LastName)
            myIpnStringBuilder.AppendLine("Buyer's Sipping Address -> City : " + e.BuyerInfo.ShippingAddress.City)
            myIpnStringBuilder.AppendLine("Subscription Event : " + e.SubscriptionInfo.Event.ToString())
            myIpnStringBuilder.AppendLine()
            myIpnStringBuilder.AppendLine(e.IPN.ToString())
            File.WriteAllText(MapPath(String.Format("~/App_Data/suscr_ipn_cancel_{0}.txt", e.SubscriptionInfo.SubscriberID)), myIpnStringBuilder.ToString())
        End If
    End Sub

    ''' <summary>
    ''' Handles the Exception event of the SubscriptionButton_IPN control. This event is fired if there is any Exception
    ''' thrown in your IPN_Notified event handler method or if there was any exception thrown in the notification
    ''' verification phase of IPN. It is better to handle this event and keep your IPN_Notified event clean without
    ''' using TRY-CATCH block there.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    Protected Sub SubscriptionButton_IPN_Exception(ByVal sender As Object, ByVal e As IPNExceptionEventArgs) Handles SubscriptionButton1.IPN_Exception
        Dim exceptionBuilder As New StringBuilder(e.IpnException.Message)
        exceptionBuilder.AppendLine("............................")
        exceptionBuilder.AppendLine("Stack Trace : ")
        exceptionBuilder.AppendLine(e.IpnException.StackTrace)
        File.WriteAllText(MapPath(String.Format("~/App_Data/subscription_ipn_err_{0}.txt", e.TransactionID)), exceptionBuilder.ToString())
    End Sub

    ''' <summary>
    ''' Handles the Returned event of the SubscriptionButton_PayPal control. If you capture PDT (by providing the PDT Authentication
    ''' token) you will get the same set of transaction data from it's event argument object that you get in IPN_Notified event.
    ''' So, you can also process the post payment/subscription logic from this event. BUT, it is never a good idea to do so. WHY ? 
    ''' Because, if your customer closes his/her browser before returning back to your website, then, this event will never get fired and the 
    ''' post payment/subscription logic will not get executed. So, you should use a combination of IPN_Notified and PayPal_Returned event. Use 
    ''' IPN_Notified event to proceed the core post payment/subscription logic (recording the transaction to database, send email, 
    ''' update membership role etc) and use PayPal_Returned event to read that data back to your customer which was recorded 
    ''' in the IPN session. OR, simply say 'Thank you for the Payment' to your customer. A Common question "What is the sequence
    ''' of the IPN_Notified and PayPal_Returned event firing ?". The answer is not straight simple. Because, IPN_Notified event 
    ''' is an Asynchronous Event fired by PayPal Notification Service depending on the queue of their IPN requests. That can be 
    ''' instant as soon as the customer clicks 'PayNow' button in PayPal website, OR few minutes after the customer returned back
    ''' to your website. So, most of the time, you will find that, IPN_Notified event is fired before PayPal_Returned event. But
    ''' that sequence is not guaranteed. 
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    Protected Sub SubscriptionButton_PayPal_Returned(ByVal sender As Object, ByVal e As SubscriptionReturnedEventArgs)
        If e.IsPaymentCancelled Then
            MultiViewMyShop.SetActiveView(ViewPaymentCancelled)
        Else
            MultiViewMyShop.SetActiveView(ViewPaymentCompleted)
            lblTransactionID.Text = e.TransactionID
            ViewState("SubscrId") = e.SubscriptionInfo.SubscriberID
            txtPDTString.Text = e.PDT.ToString()
            Select Case e.PDT.Status
                Case PayPalPDT.StatusCodes.SUCCESS
                    lblThanks.Text = String.Format("Thank you {0} {1}! for your payment.", e.BuyerInfo.FirstName, e.BuyerInfo.LastName)
                    lblSubscriptionName.Text = e.SubscriptionInfo.SubscriptionName
                    lblSubscriptionNumber.Text = e.SubscriptionInfo.SubscriptionNumber
                    lblResult.Text = e.PDT.Status.ToString()
                    lblPaymentAmount.Text = e.PaymentInfo.McGross.ToString()
                    Dim paymentButton As SubscriptionButton = DirectCast(sender, SubscriptionButton)
                    lblCustomerName.Text = paymentButton.AdditionalDataItems("Customer's Name")
                    lblSubscriptionEvent.Text = e.SubscriptionInfo.[Event].ToString()
                    lblSubscriberId.Text = e.SubscriptionInfo.SubscriberID
                    Exit Select
                Case PayPalPDT.StatusCodes.NOT_Handling_PDT
                    lblThanks.Text = "Thank you for the payment. As PDT is not being handled, " & vbCr & vbLf & "                no transaction data is available. But you will get email about your purchased" & vbCr & vbLf & "                product receipt and shipping information."
                    lblTransactionID.Text = e.TransactionID
                    Exit Select
                Case PayPalPDT.StatusCodes.NO_PDT_Available
                    lblThanks.Text = "Thank you signing up. The PDT is not available. Maybe because you had FREE Trial period " & vbCr & vbLf & "for your Subscription. If there is a FREE Trial period, no payment transaction takes place at Subscription Sign up and so" & vbCr & vbLf & "when Customer is returned to your website from PayPal, PDT is not available."
                    Exit Select
            End Select
        End If
    End Sub

    ''' <summary>
    ''' Handles the Click event of the btnReadIPNString control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    Protected Sub btnReadIPNString_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnReadIPNString.Click
        If ViewState("SubscrId") Is Nothing Then
            lblIPNReadStatus.Text = "Looks like either PDT is not being handled or there was a FREE trial where payment was not made."
            Return
        End If

        Dim filePath As String = MapPath(String.Format("~/App_Data/suscr_ipn_signup_{0}.txt", ViewState("SubscrId")))
        If File.Exists(filePath) Then
            txtIPNStr.Text = File.ReadAllText(filePath)
            Try
                File.Delete(filePath)
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder."
            Catch
                lblIPNReadStatus.Text = "IPN file could not be deleted."
            End Try
        Else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. Try again."
        End If
    End Sub

    ''' <summary>
    ''' Handles the Click event of the btnReadIPNPaymentEventData control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    Protected Sub btnReadIPNPaymentEventData_Click(ByVal sender As Object, ByVal e As EventArgs)
        If ViewState("SubscrId") Is Nothing Then
            lblIPNReadStatus.Text = "Looks like either PDT is not being handled or there was a FREE trial where payment was not made."
            Return
        End If

        Dim filePath As String = MapPath(String.Format("~/App_Data/suscr_ipn_payment_{0}.txt", ViewState("SubscrId")))
        If File.Exists(filePath) Then
            txtIPNStr.Text = File.ReadAllText(filePath)
            Try
                File.Delete(filePath)
                lblIPNReadStatus.Text = "SUCCESS! IPN file successfully read and deleted from App_Data folder."
            Catch
                lblIPNReadStatus.Text = "IPN file could not be deleted."
            End Try
        Else
            lblIPNReadStatus.Text = "IPN file could not be found in the App_Data folder. Try again."
        End If
    End Sub

End Class
